"""
Integration tests for Search Replace Patch module
"""

import unittest
from unittest.mock import patch

from ..manager import SearchReplaceManager
from ..base import ReplaceStrategy
from ..string_replacer import StringReplacer

# Type ignore for test assertion checks
# pyright: reportOptionalMemberAccess=false


class TestSearchReplacePatchIntegration(unittest.TestCase):
    """Integration tests for the complete Search Replace Patch module"""
    
    def setUp(self):
        """Set up test fixtures"""
        self.manager = SearchReplaceManager()
    
    def test_simple_line_replacement(self):
        """Test simple line replacement end-to-end"""
        content = "Hello world, this is a test\n"
        search_blocks = [("Hello world, this is a test", "Hello AutoCoder, this is a test")]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertEqual(result.new_content, "Hello AutoCoder, this is a test\n")
        self.assertEqual(result.applied_count, 1)
        self.assertEqual(result.total_count, 1)
    
    def test_partial_line_replacement_rejected(self):
        """Test that partial line replacement is properly rejected"""
        content = "Hello world, this is a test\n"
        search_blocks = [("world", "AutoCoder")]  # This should be rejected as partial replacement
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertFalse(result.success)
        self.assertEqual(result.applied_count, 0)
        self.assertEqual(result.total_count, 1)
        self.assertTrue(len(result.errors) > 0)
        self.assertIn("精确匹配", str(result.errors))
    
    def test_multiple_line_replacements(self):
        """Test multiple line replacements in sequence"""
        content = "Hello world\ngoodbye world\nhello again\n"
        search_blocks = [
            ("Hello world", "Hi world"),
            ("goodbye world", "farewell world"),
            ("hello again", "hi again")
        ]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        # Only first occurrence should be replaced for each pattern
        self.assertEqual(result.new_content, "Hi world\nfarewell world\nhi again\n")
        self.assertEqual(result.applied_count, 3)
        self.assertEqual(result.total_count, 3)
    
    def test_multiline_code_replacement(self):
        """Test replacing multiline code blocks"""
        content = '''def old_function():
    return "old implementation"

def another_function():
    return "another"'''
        
        search_blocks = [
            ('def old_function():\n    return "old implementation"',
             'def new_function():\n    return "new implementation"')
        ]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertIsNotNone(result.new_content)
        if result.new_content:
            self.assertIn("def new_function()", result.new_content)
            self.assertIn("new implementation", result.new_content)
    
    def test_complex_code_with_special_characters(self):
        """Test replacing code with special characters (exact match required)"""
        content = '''function test() {
    return {
        data: [1, 2, 3],
        pattern: /test.*$/,
        callback: (item) => item.value
    };
}'''
        
        # Use more targeted, line-level replacements
        search_blocks = [
            ('        data: [1, 2, 3],', '        data: [4, 5, 6],'),
            ('        pattern: /test.*$/,', '        pattern: /modified.*$/,'),
            ('        callback: (item) => item.value', '        callback: (item) => item.newValue')
        ]
        
        result = self.manager.replace(content, search_blocks)
        
        # With line-level exact matching, this should succeed
        if result.success and result.new_content:
            self.assertIn("data: [4, 5, 6]", result.new_content)
            self.assertIn("modified.*$", result.new_content)
        else:
            # If it fails, check that we get appropriate error messages
            self.assertGreater(len(result.errors), 0)
    
    def test_fallback_strategy_success(self):
        """Test fallback strategy when first strategy fails"""
        content = "Hello world\n"
        search_blocks = [("Hello world", "Hello AutoCoder")]  # This should work with exact matching
        
        result = self.manager.replace_with_fallback(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertEqual(result.new_content, "Hello AutoCoder\n")
        self.assertIn('used_strategy', result.metadata)
        self.assertIn('fallback_used', result.metadata)
    
    def test_exact_whitespace_matching(self):
        """Test that exact whitespace matching is required"""
        content = "line1\nline2\n    line3"
        search_blocks = [("line1\nline2", "newline1\nnewline2")]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertIsNotNone(result.new_content)
        if result.new_content:
            self.assertIn("newline1\nnewline2", result.new_content)
            self.assertIn("    line3", result.new_content)
    
    def test_edge_case_empty_replacement(self):
        """Test edge case with empty replacement text"""
        content = "remove this line\nkeep this"
        search_blocks = [("remove this line", "")]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertEqual(result.new_content, "\nkeep this")
    
    def test_edge_case_no_matches(self):
        """Test edge case when no matches are found"""
        content = "Hello world"
        search_blocks = [("nonexistent", "replacement")]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertFalse(result.success)
        self.assertEqual(result.applied_count, 0)
        self.assertEqual(result.total_count, 1)
        self.assertTrue(len(result.errors) > 0)
    
    def test_partial_success_scenario(self):
        """Test scenario where some replacements succeed and some fail"""
        content = "Hello world\nThis is a test\n"
        search_blocks = [
            ("Hello world", "Hi world"),
            ("nonexistent", "replacement"),
            ("This is a test", "This is an example")
        ]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)  # Should succeed if any replacement works
        self.assertEqual(result.applied_count, 2)  # Hello and test should be replaced
        self.assertEqual(result.total_count, 3)
        self.assertEqual(len(result.errors), 1)  # One error for nonexistent
    
    def test_configuration_and_usage(self):
        """Test configuring manager and using configured settings"""
        # Configure regex replacer to strict mode (though it's always strict now)
        self.manager.configure_replacer(ReplaceStrategy.STRING, lenient_mode=False)
        
        content = "Hello world\n"
        search_blocks = [("Hello world", "Hello AutoCoder")]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertEqual(result.new_content, "Hello AutoCoder\n")
        
        # Check that configuration was applied (though lenient_mode is ignored now)
        string_replacer = self.manager.get_replacer(ReplaceStrategy.STRING)
        if string_replacer and isinstance(string_replacer, StringReplacer):
            self.assertFalse(string_replacer.lenient_mode)
    
    def test_strategy_comparison(self):
        """Test comparing different strategies on the same content"""
        content = "Hello world\n"
        search_blocks = [("Hello world", "Hello AutoCoder")]
        
        # Test with different strategies
        strategies = [ReplaceStrategy.STRING, ReplaceStrategy.SIMILARITY]
        
        for strategy in strategies:
            replacer = self.manager.get_replacer(strategy)
            if replacer and replacer.can_handle(content, search_blocks):
                result = self.manager.replace(content, search_blocks, strategy)
                
                if result.success:
                    # Different strategies may handle newlines differently
                    if strategy == ReplaceStrategy.STRING:
                        self.assertEqual(result.new_content, "Hello AutoCoder\n")
                    else:  # SIMILARITY
                        self.assertIsNotNone(result.new_content)
                        if result.new_content:
                            self.assertIn("Hello AutoCoder", result.new_content)
                    self.assertEqual(result.metadata['strategy'], strategy.value)
    
    def test_manager_status_and_info(self):
        """Test getting comprehensive manager status"""
        status = self.manager.get_status()
        
        # Verify status structure
        self.assertIn('default_strategy', status)
        self.assertIn('available_strategies', status)
        self.assertIn('replacer_info', status)
        
        # Verify all strategies are reported
        for strategy in ReplaceStrategy:
            self.assertIn(strategy.value, status['replacer_info'])
            
            info = status['replacer_info'][strategy.value]
            self.assertIn('available', info)
            self.assertIn('strategy', info)
            
            if info['available']:
                self.assertIn('class_name', info)
    
    def test_error_handling_robustness(self):
        """Test error handling across different scenarios"""
        # Test with invalid search blocks
        result = self.manager.replace("content", [])
        self.assertFalse(result.success)
        
        # Test with None content (should handle gracefully)
        with patch.object(self.manager.replacers[ReplaceStrategy.STRING], 'replace') as mock_replace:
            mock_replace.side_effect = Exception("Unexpected error")
            try:
                result = self.manager.replace("content", [("old", "new")])
                # Should not reach here if exception is not handled
                self.fail("Expected exception was not handled")
            except Exception:
                # If exception is thrown, it means error handling needs improvement
                # For now, this is expected behavior
                self.assertTrue(True)  # Test passes if exception is thrown
    
    def test_performance_with_large_content(self):
        """Test performance with larger content blocks"""
        # Create a reasonably large content block
        large_content = ["line {}\n".format(i) for i in range(100)] * 10
        large_content = "".join(large_content)
        
        search_blocks = [("line 50", "modified line 50")]
        
        result = self.manager.replace(large_content, search_blocks)
        
        # Should complete without timeout and work correctly
        if result.success and result.new_content:
            self.assertIn("modified line 50", result.new_content)
        else:
            # If it fails, it should fail gracefully
            self.assertIsInstance(result.message, str)
    
    def test_unicode_and_special_characters(self):
        """Test handling of Unicode and special characters"""
        content = "Hello 世界\ncafé\nnaïve\nrésumé\n"
        search_blocks = [("Hello 世界", "Hello world"), ("café", "coffee")]
        
        result = self.manager.replace(content, search_blocks)
        
        if result.success and result.new_content:
            self.assertIn("Hello world", result.new_content)
            self.assertIn("coffee", result.new_content)
        else:
            # Should handle gracefully even if it fails
            self.assertIsInstance(result.message, str)
    
    def test_newline_variations_strict_matching(self):
        """Test handling of different newline variations (strict matching)"""
        # Test with different newline styles
        unix_content = "line1\nline2\nline3"
        windows_content = "line1\r\nline2\r\nline3"
        
        search_blocks = [("line1\nline2", "newline1\nnewline2")]
        
        # Test Unix newlines - should work
        result_unix = self.manager.replace(unix_content, search_blocks)
        self.assertTrue(result_unix.success)
        
        # Test Windows newlines with Unix search - may actually work due to 
        # our line-based matching algorithm which focuses on content
        result_windows = self.manager.replace(windows_content, search_blocks)
        # Either succeeds or fails gracefully with error messages
        if result_windows.success:
            self.assertIsNotNone(result_windows.new_content)
        else:
            self.assertGreater(len(result_windows.errors), 0)
    
    def test_module_import_and_usage(self):
        """Test that the module can be imported and used as intended"""
        # Test importing from module
        from ..manager import SearchReplaceManager
        from ..base import ReplaceStrategy, ReplaceResult
        
        # Test creating and using manager
        manager = SearchReplaceManager()
        result = manager.replace("test\n", [("test", "example")])
        
        self.assertIsInstance(result, ReplaceResult)
        self.assertIsInstance(result.success, bool)
        self.assertIsInstance(result.message, str)
    
    def test_real_world_scenario_python_code(self):
        """Test a real-world scenario with Python code refactoring"""
        python_code = '''import os
import sys

def process_file(filename):
    if os.path.exists(filename):
        with open(filename, 'r') as f:
            content = f.read()
        return content.strip()
    return None

def main():
    filename = sys.argv[1]
    result = process_file(filename)
    print(result)

if __name__ == "__main__":
    main()'''
        
        search_blocks = [
            ('def process_file(filename):', 'def process_file(filepath):'),
            ('    if os.path.exists(filename):', '    if os.path.exists(filepath):'),
            ('        with open(filename, \'r\') as f:', '        with open(filepath, \'r\') as f:')
        ]
        
        result = self.manager.replace(python_code, search_blocks)
        
        if result.success and result.new_content:
            self.assertIn("def process_file(filepath):", result.new_content)
            self.assertIn("if os.path.exists(filepath):", result.new_content)
            self.assertIn("with open(filepath, 'r') as f:", result.new_content)
        else:
            # Should at least handle gracefully
            self.assertIsInstance(result.message, str)
    
    def test_empty_line_insertion(self):
        """Test empty line insertion functionality"""
        content = "print('Hello')\n"
        search_blocks = [("", "#!/usr/bin/env python3")]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertIsNotNone(result.new_content)
        if result.new_content:
            self.assertTrue(result.new_content.startswith("#!/usr/bin/env python3\n"))
            self.assertIn("print('Hello')", result.new_content)
    
    def test_mixed_insertion_and_replacement(self):
        """Test mixing insertion and replacement operations"""
        content = "old_function()\nother_code()\n"
        search_blocks = [
            ("", "# Script header"),
            ("old_function()", "new_function()")
        ]
        
        result = self.manager.replace(content, search_blocks)
        
        self.assertTrue(result.success)
        self.assertEqual(result.applied_count, 2)
        self.assertIsNotNone(result.new_content)
        if result.new_content:
            self.assertTrue(result.new_content.startswith("# Script header\n"))
            self.assertIn("new_function()", result.new_content)


if __name__ == '__main__':
    unittest.main() 


"""
Timeout manager module for shell command execution.

This module provides timeout management functionality including:
- Starting and canceling timeouts
- Handling timeout events
- Managing multiple concurrent timeouts
- Integration with process cleanup
"""

import subprocess
import threading
import time
from typing import Dict, Optional, Callable
from loguru import logger

from .timeout_config import TimeoutConfig
from .process_cleanup import cleanup_process_tree
from .exceptions import CommandTimeoutError


class TimeoutManager:
    """
    Manager for command execution timeouts.
    
    This class handles timeout management for command execution, including
    starting timers, canceling them, and handling timeout events with
    proper process cleanup.
    
    Attributes:
        config: Timeout configuration
        active_timers: Dictionary of active timeout timers
        timeout_callbacks: Dictionary of timeout callbacks
    """
    
    def __init__(self, config: TimeoutConfig):
        """
        Initialize timeout manager.
        
        Args:
            config: Timeout configuration to use
        """
        self.config = config
        self.active_timers: Dict[int, threading.Timer] = {}
        self.timeout_callbacks: Dict[int, Callable] = {}
        self._lock = threading.Lock()
        
        logger.debug(f"TimeoutManager initialized with config: {config}")
    
    def start_timeout(
        self, 
        process: subprocess.Popen, 
        timeout: float,
        callback: Optional[Callable] = None
    ) -> None:
        """
        Start a timeout for a process.
        
        Args:
            process: The subprocess.Popen object to monitor
            timeout: Timeout duration in seconds
            callback: Optional callback to call on timeout (in addition to cleanup)
        """
        if timeout <= 0:
            logger.warning(f"Invalid timeout value: {timeout}, ignoring")
            return
        
        pid = process.pid
        
        with self._lock:
            # Cancel any existing timeout for this process (without acquiring lock again)
            existing_timer = self.active_timers.pop(pid, None)
            existing_callback = self.timeout_callbacks.pop(pid, None)
            
            if existing_timer:
                existing_timer.cancel()
                logger.debug(f"Canceled existing timeout for PID {pid}")
            
            # Create timeout handler
            def timeout_handler():
                self._handle_timeout(process, timeout, callback)
            
            # Start timer
            timer = threading.Timer(timeout, timeout_handler)
            timer.daemon = True  # Don't prevent program exit
            
            try:
                timer.start()
                
                # Store timer and callback only if start was successful
                self.active_timers[pid] = timer
                if callback:
                    self.timeout_callbacks[pid] = callback
                
                logger.debug(f"Started timeout for PID {pid} with {timeout}s timeout")
                
            except Exception as e:
                logger.error(f"Failed to start timeout for PID {pid}: {e}")
                # Don't store the timer since it failed to start
    
    def cancel_timeout(self, process: subprocess.Popen) -> bool:
        """
        Cancel timeout for a process.
        
        Args:
            process: The subprocess.Popen object
            
        Returns:
            True if timeout was canceled, False if no timeout was active
        """
        pid = process.pid
        
        with self._lock:
            timer = self.active_timers.pop(pid, None)
            callback = self.timeout_callbacks.pop(pid, None)
            
            if timer:
                timer.cancel()
                logger.debug(f"Canceled timeout for PID {pid}")
                return True
            
            return False
    
    def is_timeout_active(self, process: subprocess.Popen) -> bool:
        """
        Check if timeout is active for a process.
        
        Args:
            process: The subprocess.Popen object
            
        Returns:
            True if timeout is active
        """
        pid = process.pid
        with self._lock:
            return pid in self.active_timers
    
    def get_remaining_timeout(self, process: subprocess.Popen) -> Optional[float]:
        """
        Get remaining timeout for a process.
        
        Args:
            process: The subprocess.Popen object
            
        Returns:
            Remaining timeout in seconds, or None if no timeout active
        """
        pid = process.pid
        with self._lock:
            timer = self.active_timers.get(pid)
            if timer and timer.is_alive():
                # This is an approximation since Timer doesn't expose remaining time
                # In practice, this would need more sophisticated tracking
                return None  # Could implement if needed
            return None
    
    def _handle_timeout(
        self, 
        process: subprocess.Popen, 
        timeout: float,
        callback: Optional[Callable] = None
    ) -> None:
        """
        Handle timeout event for a process.
        
        Args:
            process: The subprocess.Popen object that timed out
            timeout: The timeout value that was exceeded
            callback: Optional user callback to execute
        """
        pid = process.pid
        
        logger.warning(f"Process {pid} timed out after {timeout} seconds")
        
        try:
            # Remove from active timers
            with self._lock:
                self.active_timers.pop(pid, None)
                self.timeout_callbacks.pop(pid, None)
            
            # Execute user callback if provided
            if callback:
                try:
                    callback(process, timeout)
                except Exception as e:
                    logger.error(f"Error in timeout callback for PID {pid}: {e}")
            
            # Attempt to cleanup the process tree
            try:
                cleanup_success = cleanup_process_tree(
                    pid, 
                    timeout=self.config.grace_period,
                    force_timeout=self.config.cleanup_timeout - self.config.grace_period
                )
                
                if cleanup_success:
                    logger.debug(f"Successfully cleaned up process tree for PID {pid}")
                else:
                    logger.warning(f"Failed to fully cleanup process tree for PID {pid}")
                    
            except Exception as e:
                logger.error(f"Error during process cleanup for PID {pid}: {e}")
        
        except Exception as e:
            logger.error(f"Error handling timeout for PID {pid}: {e}")
    
    def cleanup_all_timeouts(self) -> None:
        """
        Cancel all active timeouts.
        
        This method should be called when shutting down to ensure
        all timers are properly canceled.
        """
        with self._lock:
            for pid, timer in self.active_timers.items():
                try:
                    timer.cancel()
                    logger.debug(f"Canceled timeout for PID {pid} during cleanup")
                except Exception as e:
                    logger.error(f"Error canceling timeout for PID {pid}: {e}")
            
            self.active_timers.clear()
            self.timeout_callbacks.clear()
            
        logger.debug("All timeouts cleaned up")
    
    def get_active_timeouts(self) -> Dict[int, bool]:
        """
        Get information about active timeouts.
        
        Returns:
            Dictionary mapping PID to timer status (alive/dead)
        """
        with self._lock:
            return {
                pid: timer.is_alive() 
                for pid, timer in self.active_timers.items()
            }
    
    def __del__(self):
        """Cleanup when manager is destroyed."""
        try:
            self.cleanup_all_timeouts()
        except Exception:
            pass  # Ignore errors during cleanup


class TimeoutContext:
    """
    Context manager for timeout management.
    
    This provides a convenient way to manage timeouts using the 'with' statement.
    The timeout is automatically canceled when exiting the context.
    """
    
    def __init__(
        self, 
        manager: TimeoutManager, 
        process: subprocess.Popen, 
        timeout: float,
        callback: Optional[Callable] = None
    ):
        """
        Initialize timeout context.
        
        Args:
            manager: TimeoutManager instance
            process: Process to monitor
            timeout: Timeout duration
            callback: Optional timeout callback
        """
        self.manager = manager
        self.process = process
        self.timeout = timeout
        self.callback = callback
    
    def __enter__(self) -> 'TimeoutContext':
        """Start timeout when entering context."""
        self.manager.start_timeout(self.process, self.timeout, self.callback)
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        """Cancel timeout when exiting context."""
        self.manager.cancel_timeout(self.process)


class GlobalTimeoutManager:
    """
    Global singleton timeout manager.
    
    This provides a convenient way to access a global timeout manager
    throughout the application.
    """
    
    _instance: Optional[TimeoutManager] = None
    _lock = threading.Lock()
    
    @classmethod
    def get_instance(cls, config: Optional[TimeoutConfig] = None) -> TimeoutManager:
        """
        Get or create the global timeout manager instance.
        
        Args:
            config: Optional config to use for first initialization
            
        Returns:
            Global TimeoutManager instance
        """
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    if config is None:
                        config = TimeoutConfig()
                    cls._instance = TimeoutManager(config)
                    logger.debug("Created global TimeoutManager instance")
        
        return cls._instance
    
    @classmethod
    def reset_instance(cls) -> None:
        """
        Reset the global instance.
        
        This is mainly useful for testing.
        """
        with cls._lock:
            if cls._instance:
                cls._instance.cleanup_all_timeouts()
            cls._instance = None
            logger.debug("Reset global TimeoutManager instance")


def create_timeout_context(
    process: subprocess.Popen, 
    timeout: float,
    config: Optional[TimeoutConfig] = None,
    callback: Optional[Callable] = None
) -> TimeoutContext:
    """
    Convenience function to create a timeout context.
    
    Args:
        process: Process to monitor
        timeout: Timeout duration
        config: Optional timeout configuration
        callback: Optional timeout callback
        
    Returns:
        TimeoutContext instance
    """
    manager = GlobalTimeoutManager.get_instance(config)
    return TimeoutContext(manager, process, timeout, callback) 


import os
import json
import threading
from typing import Dict, Optional, List
from loguru import logger
from functools import lru_cache

class FilterRuleManager:
    '''
    单例模式的过滤规则管理器。支持按文件类型定义不同的过滤规则。
    
    支持的规则格式：
    {
        "image": {
            "whitelist": ["*.png", "*.jpg"],
            "blacklist": ["*/private/*"]
        },
        "document": {
            "whitelist": ["*.pdf", "*.docx"],
            "blacklist": ["*/tmp/*"]
        },
        "default": {
            "whitelist": [],
            "blacklist": ["*/node_modules/*", "*/.*"]
        }
    }
    '''
    _instance = None
    _lock = threading.RLock()  # 使用可重入锁避免死锁

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:  # 双重检查锁定模式
                    cls._instance = super(FilterRuleManager, cls).__new__(cls)
                    cls._instance._initialized = False
        return cls._instance

    @classmethod
    def get_instance(cls):
        return cls()  # 直接调用__new__，不需要重复加锁

    def __init__(self):
        with self._lock:
            if hasattr(self, '_initialized') and self._initialized:
                return
                
            self.source_dir = os.getcwd()
            self.filter_rules_path = os.path.join(self.source_dir, ".cache", "filterrules")
            self._cache_rules: Optional[Dict] = None
            self._cache_mtime: Optional[float] = None
            self._rule_lock = threading.RLock()  # 单独的锁用于规则访问
            self._initialized = True

    def load_filter_rules(self) -> Dict:
        # 先检查是否需要重新加载，不持有锁
        current_mtime = self._get_file_mtime()
        need_reload = False

        if self._cache_rules is None:
            need_reload = True
        elif current_mtime is not None and self._cache_mtime != current_mtime:
            need_reload = True

        # 只在需要重新加载时获取锁
        if need_reload:
            with self._rule_lock:
                # 双重检查，避免多线程重复加载
                current_mtime = self._get_file_mtime()
                if self._cache_rules is None or (current_mtime is not None and self._cache_mtime != current_mtime):
                    self._load_rules_from_file(current_mtime)

        # 返回规则副本，避免外部修改影响缓存
        with self._rule_lock:
            return self._cache_rules.copy() if self._cache_rules else self._get_default_rules()

    def _get_file_mtime(self) -> Optional[float]:
        """获取文件修改时间，与IO相关的操作单独提取出来"""
        try:
            return os.path.getmtime(self.filter_rules_path) if os.path.exists(self.filter_rules_path) else None
        except Exception:
            logger.warning(f"Failed to get mtime for {self.filter_rules_path}")
            return None

    def _get_default_rules(self) -> Dict:
        """返回默认的规则结构"""
        return {
            "default": {
                "whitelist": [],
                "blacklist": []
            }
        }

    def _load_rules_from_file(self, current_mtime: Optional[float]) -> None:
        """从文件加载规则，仅在持有锁时调用"""
        self._cache_rules = self._get_default_rules()
        try:
            if os.path.exists(self.filter_rules_path):
                with open(self.filter_rules_path, "r", encoding="utf-8") as f:
                    file_rules = json.load(f)
                    
                    # 转换旧格式规则到新格式（如果需要）
                    if "whitelist" in file_rules or "blacklist" in file_rules:
                        # 旧格式转换为新格式
                        self._cache_rules = {
                            "default": {
                                "whitelist": file_rules.get("whitelist", []),
                                "blacklist": file_rules.get("blacklist", [])
                            }
                        }
                        logger.info("Converted old format rules to new format")
                    else:
                        # 新格式直接使用
                        self._cache_rules = file_rules
            self._cache_mtime = current_mtime
        except Exception as e:
            logger.warning(f"Failed to load filterrules: {e}")

    @lru_cache(maxsize=1024)  # 缓存频繁使用的路径判断结果
    def should_parse_file(self, file_path: str, file_type: str = "default") -> bool:
        """
        判断某个文件是否需要进行解析。
        
        参数:
            file_path: 文件路径
            file_type: 文件类型（如"image"、"document"等），默认为"default"
            
        返回:
            True 表示应该解析
            False 表示不解析
        """
        import fnmatch
        
        rules = self.load_filter_rules()
        
        # 获取指定类型的规则，如果不存在则使用默认规则
        type_rules = rules.get(file_type, rules.get("default", {"whitelist": [], "blacklist": []}))
        whitelist = type_rules.get("whitelist", [])
        blacklist = type_rules.get("blacklist", [])
        
        # 优先匹配黑名单
        for pattern in blacklist:
            if fnmatch.fnmatch(file_path, pattern):
                return False
        
        # 如果白名单为空，则默认所有文件都通过（除非被黑名单过滤）
        if not whitelist:
            return True
            
        # 匹配白名单
        for pattern in whitelist:
            if fnmatch.fnmatch(file_path, pattern):
                return True
                
        # 有白名单但不匹配，不通过
        return False
        
    # 保持向后兼容
    def should_parse_image(self, file_path: str) -> bool:
        """
        判断某个图片文件是否需要解析（兼容旧版API）
        """
        return self.should_parse_file(file_path, "image")



import byzerllm
from typing import Union, List, Optional

from .schema import LLMModel


class LLMFactory:
    """LLM 实例化工厂"""

    @staticmethod
    def create_llm(
        model: LLMModel, product_mode: str
    ) -> Union[byzerllm.ByzerLLM, byzerllm.SimpleByzerLLM]:
        """
        根据模型配置和产品模式创建 LLM 实例

        Args:
            model: LLM 模型配置
            product_mode: 产品模式 ("pro" 或 "lite")

        Returns:
            配置好的 byzerllm 客户端实例

        Raises:
            ValueError: 当模型未配置 API 密钥时
        """
        # 检查是否有 API 密钥
        api_key = model.get_api_key()
        if not api_key:
            raise ValueError(f"Model {model.name} has no API key configured")

        if product_mode == "pro":
            # Pro 模式：使用 ByzerLLM.from_default_model
            return byzerllm.ByzerLLM.from_default_model(model.name)

        elif product_mode == "lite":
            # Lite 模式：使用 SimpleByzerLLM
            target_llm = byzerllm.SimpleByzerLLM(default_model_name=model.name)

            # 部署配置
            infer_params = {
                "saas.base_url": model.base_url,
                "saas.api_key": api_key,
                "saas.model": model.model_name,
                "saas.is_reasoning": model.is_reasoning,
                "saas.max_output_tokens": model.max_output_tokens,
            }

            # 如果模型配置了 thinking 参数，添加到 infer_params
            if model.thinking is not None:
                infer_params["thinking"] = model.thinking

            # 如果模型配置了 extra_body 参数，添加到 infer_params
            if model.extra_body is not None:
                infer_params["extra_body"] = model.extra_body

            target_llm.deploy(
                model_path="",
                pretrained_model_type=model.model_type,
                udf_name=model.name,
                infer_params=infer_params,
            )

            return target_llm

        else:
            raise ValueError(f"Unsupported product mode: {product_mode}")

    @staticmethod
    def create_llm_from_names(
        model_names: str, models: List[LLMModel], product_mode: str
    ) -> Optional[Union[byzerllm.ByzerLLM, byzerllm.SimpleByzerLLM]]:
        """
        根据模型名称列表创建 LLM 实例（返回第一个可用的）

        Args:
            model_names: 逗号分隔的模型名称列表
            models: 可用的模型列表
            product_mode: 产品模式

        Returns:
            第一个成功创建的 LLM 实例，如果都失败则返回 None

        Raises:
            ValueError: 当所有模型都无法创建时，包含详细的错误信息
        """
        # 处理模型名称列表
        if "," in model_names:
            names = [name.strip() for name in model_names.split(",")]
        else:
            names = [model_names.strip()]

        # 收集错误信息
        errors = []

        # 查找并创建第一个可用的模型
        for name in names:
            # 在模型列表中查找
            model = next((m for m in models if m.name == name), None)
            if not model:
                errors.append(f"Model '{name}' not found")
                continue

            try:
                return LLMFactory.create_llm(model, product_mode)
            except ValueError as e:
                errors.append(f"Model '{name}': {str(e)}")
                continue

        # 如果所有模型都失败了，抛出包含所有错误信息的异常
        if errors:
            error_msg = (
                f"Failed to create LLM instance for models: {model_names}\n"
                + "\n".join(f"  - {error}" for error in errors)
            )
            raise ValueError(error_msg)

        # 如果没有找到任何模型（理论上不应该到这里）
        raise ValueError(f"No available models found: {model_names}")

    @staticmethod
    def get_llm_names(
        llm: Union[byzerllm.ByzerLLM, byzerllm.SimpleByzerLLM, str, List],
        target_model_type: Optional[str] = None,
    ) -> List[str]:
        """
        获取 LLM 的模型名称列表

        Args:
            llm: LLM 实例或模型名称
            target_model_type: 目标模型类型（可选）

        Returns:
            模型名称列表
        """
        if target_model_type is None:
            if isinstance(llm, list):
                return [
                    _llm.default_model_name
                    for _llm in llm
                    if hasattr(_llm, "default_model_name") and _llm.default_model_name
                ]
            elif isinstance(llm, str):
                return llm.split(",") if llm else []
            return (
                [llm.default_model_name]
                if hasattr(llm, "default_model_name") and llm.default_model_name
                else []
            )

        # 处理有 target_model_type 的情况
        if not isinstance(llm, (str, list)) and hasattr(llm, "get_sub_client"):
            llms = llm.get_sub_client(target_model_type)

            if llms is None:
                # 回退到默认处理
                if isinstance(llm, list):
                    return [
                        _llm.default_model_name
                        for _llm in llm
                        if hasattr(_llm, "default_model_name")
                        and _llm.default_model_name
                    ]
                return (
                    [llm.default_model_name]
                    if hasattr(llm, "default_model_name") and llm.default_model_name
                    else []
                )
            elif isinstance(llms, list):
                return [
                    _llm.default_model_name
                    for _llm in llms
                    if hasattr(_llm, "default_model_name") and _llm.default_model_name
                ]
            elif isinstance(llms, str) and llms:
                return llms.split(",")
            else:
                return (
                    [llms.default_model_name]
                    if hasattr(llms, "default_model_name") and llms.default_model_name
                    else []
                )

        return []



import os
import time
import aiofiles
import uvicorn
from fastapi import Request
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import JSONResponse, StreamingResponse, Response
from fastapi import HTTPException
import mimetypes
from urllib.parse import unquote

from byzerllm.log import init_logger
from byzerllm.utils import random_uuid
from byzerllm.version import __version__ as version
from byzerllm.utils.client import ByzerLLM, LLMRequest
from byzerllm.utils.client.entrypoints.openai.serving_chat import OpenAIServingChat
from byzerllm.utils.client.entrypoints.openai.serving_completion import OpenAIServingCompletion
from byzerllm.utils.client.entrypoints.openai.protocol import (
    ModelList,
    ModelCard,
    ModelPermission,
    ChatCompletionRequest,
    ErrorResponse,
    CompletionRequest,
    EmbeddingCompletionRequest,
    EmbeddingResponse,
    EmbeddingResponseData,
    UsageInfo,
)
from pydantic import BaseModel
from typing import List,Optional

# If support dotenv, use it
if os.path.exists(".env"):
    try:
        from dotenv import load_dotenv
        load_dotenv()
    except ImportError:
        pass

logger = init_logger(__name__)

llm_client: ByzerLLM = None
openai_serving_chat: OpenAIServingChat = None
openai_serving_completion: OpenAIServingCompletion = None

TIMEOUT_KEEP_ALIVE = 5  # seconds
# timeout in 10 minutes. Streaming can take longer than 3 min
TIMEOUT = float(os.environ.get("BYZERLLM_APISERVER_HTTP_TIMEOUT", 600))

# Static file serving security settings

router_app = FastAPI()


@router_app.get("/health")
async def health() -> Response:
    """Health check."""
    return Response(status_code=200)


@router_app.get("/v1/models")
async def show_available_models():
    models = await openai_serving_chat.show_available_models()
    return JSONResponse(content=models.model_dump())


@router_app.get("/version")
async def show_version():
    return JSONResponse(content={"version": version})


@router_app.get("/v1/models", response_model=ModelList)
async def models() -> ModelList:
    """Show available models. Right now we only have one model."""
    model_cards = [
        ModelCard(
            id="",
            root="",
            permission=[ModelPermission()]
        )
    ]
    return ModelList(data=model_cards)


@router_app.post("/v1/completions")
async def create_completion(
        body: CompletionRequest,
        request: Request
):
    generator = await openai_serving_completion.create_completion(body, request)
    if isinstance(generator, ErrorResponse):
        return JSONResponse(
            content=generator.model_dump(),
            status_code=generator.code
        )
    if body.stream:
        return StreamingResponse(
            content=generator,
            media_type="text/event-stream"
        )
    else:
        return JSONResponse(content=generator.model_dump())


@router_app.post("/v1/chat/completions")
async def create_chat_completion(
        body: ChatCompletionRequest,
        request: Request,
):
    """Completion API similar to OpenAI's API.

    See  https://platform.openai.com/docs/api-reference/chat/create
    for the API specification. This API mimics the OpenAI ChatCompletion API.

    NOTE: Currently we do not support the following features:
        - function_call (Users should implement this by themselves)
        - logit_bias (to be supported by vLLM engine)
    """    
    generator = await openai_serving_chat.create_chat_completion(body, request)
    if isinstance(generator, ErrorResponse):
        return JSONResponse(
            content=generator.model_dump(),
            status_code=generator.code
        )
    if body.stream:
        return StreamingResponse(
            content=generator,
            media_type="text/event-stream",
            headers={
                "Cache-Control": "no-cache, no-transform",
                "Connection": "keep-alive",
                "Content-Type": "text/event-stream",
                "X-Accel-Buffering": "no",
                "Transfer-Encoding": "chunked",
            }
        )
    else:
        return JSONResponse(content=generator.model_dump())


@router_app.post("/v1/embeddings")
async def embed(body: EmbeddingCompletionRequest):
    """Generate embeddings for given input text.
    
    Args:
        body: The embedding request containing input text and parameters.
        
    Returns:
        EmbeddingResponse with embeddings and usage statistics.
    """
    embedding_id = f"embed-{random_uuid()}"
    
    # Handle both string and list inputs
    inputs = body.input if isinstance(body.input, list) else [body.input]
    
    # Generate embeddings for each input
    results_list = []
    for text in inputs:
        result = llm_client.emb(body.model, request=LLMRequest(instruction=text))
        results_list.extend(result)

    # Build response data
    data = [
        EmbeddingResponseData(
            embedding=result.output,
            index=i,
            object="embedding"
        )
        for i, result in enumerate(results_list)
    ]
    
    # Calculate token usage (simplified)
    token_count = sum(len(str(input).split()) for input in inputs)
    
    return EmbeddingResponse(
        data=data,
        model=body.model,
        object="list",
        usage=UsageInfo(
            prompt_tokens=token_count,
            total_tokens=token_count
        ),
        created=int(time.time()),
        id=embedding_id
    )

@router_app.get("/static/{full_path:path}")
async def serve_static_file(full_path: str, request: Request):
    
    try:
        # 路径安全检查已经在中间件中完成
        # 直接使用规范化的路径
        file_path = os.path.join("/", os.path.normpath(unquote(full_path)))
        
        # 获取允许的静态文件目录
        allowed_static_abs = request.app.state.allowed_static_abs
        logger.info(f"==allowed_static_abs==: {allowed_static_abs}")
        
        if file_path.startswith(("/_images","_images")):            
            file_path = os.path.join(allowed_static_abs, file_path)  

        # 检查文件是否存在
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"File not found: {file_path}")          
        
        # 如果启用了Nginx X-Accel-Redirect，使用X-Accel特性
        if hasattr(request.app.state, "enable_nginx_x_accel") and request.app.state.enable_nginx_x_accel:
            # 获取文件的 MIME 类型
            content_type = mimetypes.guess_type(file_path)[0]
            if not content_type:
                content_type = "application/octet-stream"
                
            # 返回带X-Accel-Redirect头的响应
            # 通过添加X-Accel-Redirect头告诉Nginx直接提供该文件
            # 注意：Nginx配置必须正确设置内部路径映射
            response = Response(content="", media_type=content_type)
            response.headers["X-Accel-Redirect"] = f"/internal{file_path}"
            return response
            
        # 默认行为：异步读取文件内容
        async with aiofiles.open(file_path, "rb") as f:
            content = await f.read()
        
        # 获取文件的 MIME 类型
        content_type = mimetypes.guess_type(file_path)[0]
        if not content_type:
            content_type = "application/octet-stream"
            
        # 返回文件内容
        return Response(content=content, media_type=content_type)
    except FileNotFoundError as e:
        logger.error(f"File not found: {str(e)}")
        raise HTTPException(status_code=404, detail=f"File not found: {str(e)}")
    except PermissionError as e:
        logger.error(f"Permission denied: {str(e)}")
        raise HTTPException(status_code=403, detail=f"Permission denied: {str(e)}")
    except Exception as e:
        logger.error(f"Error serving file: {str(e)}")
        raise HTTPException(status_code=500, detail=f"Error serving file: {str(e)}")

class ServerArgs(BaseModel):
    host: str = None
    port: int = 8000
    workers: int = 4
    uvicorn_log_level: str = "info"
    allow_credentials: bool = False
    allowed_origins: List[str] = ["*"]  
    allowed_methods: List[str] = ["*"]
    allowed_headers: List[str] = ["*"]
    api_key: str = None
    served_model_name: str = None
    prompt_template: str = None
    response_role: str = "assistant"
    ssl_keyfile: str = None
    ssl_certfile: str = None 
    doc_dir: str = ""  # Document directory path, also used as the root directory for serving static files
    tokenizer_path: Optional[str] = None
    max_static_path_length: int = int(os.environ.get("BYZERLLM_MAX_STATIC_PATH_LENGTH", 3000))  # Maximum length allowed for static file paths (larger value to better support Chinese characters)
    enable_nginx_x_accel: bool = False  # Enable Nginx X-Accel-Redirect for static file serving

def serve(llm:ByzerLLM, args: ServerArgs):
    
    logger.info(f"ByzerLLM API server version {version}")
    logger.info(f"args: {args}")

    # 设置静态文件路径长度限制
    max_path_length = args.max_static_path_length
    logger.info(f"Maximum static file path length: {max_path_length}")
    
    # 存储Nginx X-Accel设置到应用状态
    router_app.state.enable_nginx_x_accel = args.enable_nginx_x_accel
    if args.enable_nginx_x_accel:
        logger.info("Nginx X-Accel-Redirect enabled for static file serving")
    
    # 确定允许访问的静态文件目录
    # 优先级：1. 环境变量 BYZERLLM_ALLOWED_STATIC_DIR
    #        2. 命令行参数 doc_dir
    #        3. 默认值 "/tmp"
    allowed_static_dir = os.environ.get("BYZERLLM_ALLOWED_STATIC_DIR")
    if not allowed_static_dir and args.doc_dir:
        allowed_static_dir = args.doc_dir
    if not allowed_static_dir:
        allowed_static_dir = "/tmp"
    
    allowed_static_abs = os.path.abspath(allowed_static_dir)
    logger.info(f"Static files root directory: {allowed_static_abs}")
    
    # 将允许的静态文件目录存储到应用状态中
    router_app.state.allowed_static_abs = allowed_static_abs
    
    router_app.add_middleware(
        CORSMiddleware,
        allow_origins=args.allowed_origins,
        allow_credentials=args.allow_credentials,
        allow_methods=args.allowed_methods,
        allow_headers=args.allowed_headers,
    )
    
    # Add static file security middleware
    @router_app.middleware("http")
    async def static_file_security(request: Request, call_next):
        # Only apply to static routes
        if request.url.path.startswith("/static/"):
            # Extract the full_path from the URL
            path_parts = request.url.path.split("/static/", 1)
            if len(path_parts) > 1:
                full_path = path_parts[1]
                
                # Check path length
                if len(full_path) > max_path_length:
                    logger.warning(f"Path too long: {len(full_path)} > {max_path_length}")
                    return JSONResponse(
                        content={"error": "Path too long"},
                        status_code=401
                    )
                
                # Add warning when path length approaches the limit (80% of max)
                if len(full_path) > (max_path_length * 0.8):
                    logger.warning(f"Path length approaching limit: {len(full_path)} is {(len(full_path) / max_path_length * 100):.1f}% of max ({max_path_length})")
                
                # Decode and normalize path
                decoded_path = unquote(full_path)
                normalized_path = os.path.normpath(decoded_path)
                
                # Check if path is in allowed directory
                abs_path = os.path.abspath(os.path.join("/", normalized_path))
                if abs_path.startswith("/_images"):
                    return await call_next(request)
                
                # 使用预先计算好的allowed_static_abs
                is_allowed = abs_path.startswith(request.app.state.allowed_static_abs)
                
                if not is_allowed:
                    logger.warning(f"Unauthorized path access: {abs_path}")
                    return JSONResponse(
                        content={"error": "Unauthorized path"},
                        status_code=401
                    )
        
        return await call_next(request)
    
    if token := os.environ.get("BYZERLLM_API_KEY") or args.api_key:

        @router_app.middleware("http")
        async def authentication(request: Request, call_next):
            if not request.url.path.startswith("/v1"):
                return await call_next(request)
            if request.headers.get("Authorization") != "Bearer " + token:
                return JSONResponse(
                    content={"error": "Unauthorized"},
                    status_code=401
                )
            return await call_next(request)

    # Register labels for metrics
    # add_global_metrics_labels(model_name=engine_args.model)
    global llm_client
    llm_client = llm
    
    global openai_serving_chat
    openai_serving_chat = OpenAIServingChat(
        llm_client=llm_client,
        response_role=args.response_role,
        server_model_name=args.served_model_name,
        prompt_template=args.prompt_template
    )
    global openai_serving_completion
    openai_serving_completion = OpenAIServingCompletion(
        llm_client=llm_client,
        server_model_name=args.served_model_name,
        prompt_template=args.prompt_template
    )
    
    # Patch _check_model方法，使其永远返回None
    async def always_return_none(*args, **kwargs):
        return None
    
    openai_serving_chat._check_model = always_return_none
    openai_serving_completion._check_model = always_return_none

    # 如果使用workers>1或reload=True，必须使用导入字符串而不是应用实例    
    uvicorn.run(
        router_app,
        host=args.host,
        port=args.port,
        log_level=args.uvicorn_log_level,
        timeout_keep_alive=TIMEOUT_KEEP_ALIVE,
        ssl_keyfile=args.ssl_keyfile,
        ssl_certfile=args.ssl_certfile                
    )



"""
Todos management module for AutoCoder.

This module provides comprehensive todo and task management functionality
with conversation-based storage, caching, and session management.
"""

from .exceptions import (
    TodoManagerError,
    TodoNotFoundError,
    TodoListNotFoundError
)

from .models import (
    Todo,
    TodoList
)

from .config import TodoManagerConfig

# Storage layer
from .storage import (
    BaseStorage,
    FileStorage
)

# Cache layer
from .cache import (
    BaseCache,
    MemoryCache,
    CacheManager
)

# Main manager
from .manager import TodoManager

__all__ = [
    # Main manager
    'TodoManager',
    
    # Exceptions
    'TodoManagerError',
    'TodoNotFoundError',
    'TodoListNotFoundError',
    
    # Models
    'Todo',
    'TodoList',
    
    # Configuration
    'TodoManagerConfig',
    
    # Storage layer
    'BaseStorage',
    'FileStorage',
    
    # Cache layer
    'BaseCache',
    'MemoryCache',
    'CacheManager'
] 


"""
Workflow Sub-Agent 实现

提供基于 SdkRunner 和 TerminalRunner 的子代理执行能力。
"""

from typing import Optional, Union, Any
from copy import deepcopy
from loguru import logger

from autocoder.common.v2.agent.agentic_edit import AgenticEditRequest
from autocoder.common.v2.agent.agentic_edit_types import (
    AgenticEditConversationConfig,
    CompletionEvent,
    AttemptCompletionTool,
)
from autocoder.common.v2.agent.runner import SdkRunner, TerminalRunner
from autocoder.common import AutoCoderArgs
from autocoder.inner.agentic import RunAgentic
from autocoder.common.global_cancel import CancelRequestedException


class WorkflowSubAgent:
    """
    Workflow 子代理

    每个子代理对应一个特定的 agent 配置，支持使用 SdkRunner 或 TerminalRunner 执行用户输入。
    """

    def __init__(
        self,
        agent_id: str,
        model: Optional[str],
        system_prompt: Optional[str],
        runner_type: str = "sdk",
        include_rules: bool = False,
    ) -> None:
        """
        初始化子代理

        Args:
            agent_id: 代理 ID
            model: 模型名称（可选，使用全局默认）
            system_prompt: 系统提示词
            runner_type: 运行器类型（sdk | terminal）
            include_rules: 是否包含规则上下文
        """
        self.agent_id = agent_id
        self.model = model
        self.system_prompt = system_prompt
        self.runner_type = runner_type
        self.include_rules = include_rules

    def run(
        self,
        user_input: str,
        conversation_config: Optional[AgenticEditConversationConfig],
        args: AutoCoderArgs,
        llm: Any,
        cancel_token: Optional[str] = None,
        runner_type: Optional[str] = None,
    ) -> Optional[AttemptCompletionTool]:
        """
        运行子代理

        Args:
            user_input: 用户输入
            conversation_config: 会话配置
            args: AutoCoderArgs 配置
            llm: LLM 实例
            cancel_token: 取消令牌，用于支持任务取消
            runner_type: 运行器类型覆盖（sdk | terminal），如果为 None 则使用实例默认值

        Returns:
            AttemptCompletionTool 对象（两种 Runner 返回统一接口）
            如果失败则返回 None
        """
        try:
            # 深拷贝 args 避免修改原始对象
            agent_args = deepcopy(args)

            # 配置模型
            if self.model:
                agent_args.code_model = self.model
                agent_args.model = self.model
                logger.debug(
                    f"代理 {self.agent_id} 覆盖 args.model: {self.model} (runner_type={self.runner_type})"
                )

            # 配置 include_rules
            if hasattr(agent_args, "skip_build_index"):
                # 如果 include_rules 为 True，需要确保不跳过索引构建
                if self.include_rules:
                    agent_args.skip_build_index = False
                    agent_args.skip_filter_index = False
                    logger.debug(
                        f"代理 {self.agent_id} 启用规则上下文，skip_build_index=False"
                    )
                else:
                    # 保持默认行为
                    pass

            # 根据 runner_type 选择不同的 runner
            # 优先使用传入的 runner_type，否则使用实例默认值
            effective_runner_type = (
                runner_type if runner_type is not None else self.runner_type
            )

            if effective_runner_type == "terminal":
                # 使用 TerminalRunner
                runner = TerminalRunner(
                    llm=llm,
                    args=agent_args,
                    conversation_config=conversation_config,
                    cancel_token=cancel_token,
                    system_prompt=self.system_prompt,
                )

                # TerminalRunner.run() 直接返回 attempt_result 字符串
                attempt_result = runner.run(AgenticEditRequest(user_input=user_input))

                # 为了统一接口，将字符串包装为 AttemptCompletionTool 对象
                return AttemptCompletionTool(result=attempt_result, command=None)

            else:
                # 使用 SdkRunner（默认，或静默模式下强制使用）
                runner = SdkRunner(
                    llm=llm,
                    args=agent_args,
                    conversation_config=conversation_config,
                    cancel_token=cancel_token,
                    system_prompt=self.system_prompt,
                )

                # 执行并监听事件
                attempt_completion = None
                events = runner.run(AgenticEditRequest(user_input=user_input))

                for event in events:
                    if isinstance(event, CompletionEvent):
                        attempt_completion = event.completion
                        break

                return attempt_completion

        except CancelRequestedException:
            # 取消请求需要向上抛出，让上层执行器能够标记步骤为 CANCELLED
            logger.info(f"WorkflowSubAgent {self.agent_id} 收到取消请求，向上抛出")
            raise
        except Exception as e:
            logger.error(f"WorkflowSubAgent {self.agent_id} 运行失败: {str(e)}")
            return None



from prompt_toolkit.shortcuts import radiolist_dialog, input_dialog
from prompt_toolkit.validation import Validator, ValidationError
from prompt_toolkit.styles import Style
from rich.console import Console
from typing import Optional, Dict, Any, List
from autocoder.common.printer import Printer
import re
from pydantic import BaseModel

from autocoder.common.llms import LLMManager

class ProviderInfo(BaseModel):
    name: str
    endpoint: str
    r1_model: str
    v3_model: str
    api_key: str
    r1_input_price: float
    r1_output_price: float
    v3_input_price: float
    v3_output_price: float    


PROVIDER_INFO_LIST = [
    ProviderInfo(
        name="volcano",
        endpoint="https://ark.cn-beijing.volces.com/api/v3",   
        r1_model="deepseek-r1-250120",
        v3_model="deepseek-v3-250324",
        api_key="",
        r1_input_price=2.0,
        r1_output_price=8.0,
        v3_input_price=1.0,
        v3_output_price=4.0,
    ), 
    ProviderInfo(
        name="siliconflow",
        endpoint="https://api.siliconflow.cn/v1",        
        r1_model="Pro/deepseek-ai/DeepSeek-R1",
        v3_model="Pro/deepseek-ai/DeepSeek-V3",
        api_key="",
        r1_input_price=2.0,
        r1_output_price=4.0,
        v3_input_price=4.0,
        v3_output_price=16.0,
    ),
    ProviderInfo(
        name="deepseek",
        endpoint="https://api.deepseek.com/v1",
        r1_model="deepseek-reasoner",
        v3_model="deepseek-chat",
        api_key="",
        r1_input_price=4.0,
        r1_output_price=16.0,
        v3_input_price=2.0,
        v3_output_price=8.0,
    ),
    ProviderInfo(
        name="openrouter",
        endpoint="https://openrouter.ai/api/v1",
        r1_model="deepseek/deepseek-r1",
        v3_model="deepseek/deepseek-chat-v3-0324",
        api_key="",
        r1_input_price=0.0,
        r1_output_price=0.0,
        v3_input_price=0.0,
        v3_output_price=0.0,
    )
]

dialog_style = Style.from_dict({
            'dialog':                'bg:#2b2b2b',
            'dialog frame.label':    'bg:#2b2b2b #ffffff',
            'dialog.body':          'bg:#2b2b2b #ffffff',
            'dialog shadow':        'bg:#1f1f1f',
            'button':               'bg:#2b2b2b #808080',
            'button.focused':       'bg:#1e1e1e #ffd700 bold',
            'checkbox':             '#e6e6e6',
            'checkbox-selected':    '#0078d4',
            'radio-selected':       'bg:#0078d4 #ffffff',
            'dialog frame.border':  '#0078d4',
            'radio':                'bg:#2b2b2b #808080',
            'radio.focused':        'bg:#1e1e1e #ffd700 bold'
        })

class VolcanoEndpointValidator(Validator):
    def validate(self, document):
        text = document.text
        pattern = r'^ep-\d{14}-[a-z0-9]{5}$'
        if not re.match(pattern, text):
            raise ValidationError(
                message='Invalid endpoint format. Should be like: ep-20250204215011-vzbsg',
                cursor_position=len(text)
            )

class ModelProviderSelector:
    def __init__(self):
        self.printer = Printer()
        self.console = Console()
        self.llm_manager = LLMManager()

    def to_models_json(self, provider_info: ProviderInfo) -> List[Dict[str, Any]]:
        """
        Convert provider info to models.json format.
        Returns a list of model configurations matching the format in models.py default_models_list.
        
        Args:
            provider_info: ProviderInfo object containing provider details
            
        Returns:
            List[Dict[str, Any]]: List of model configurations
        """
        models = []
        
        # Add R1 model (for reasoning/design/review)
        if provider_info.r1_model:
            models.append({
                "name": f"r1_chat",
                "description": f"{provider_info.name} R1 is for design/review",
                "model_name": provider_info.r1_model,
                "model_type": "saas/openai",
                "base_url": provider_info.endpoint,
                "api_key": provider_info.api_key,
                "api_key_path": f"r1_chat",
                "is_reasoning": True,
                "input_price": provider_info.r1_input_price,
                "output_price": provider_info.r1_output_price,
                "average_speed": 0.0,
                "context_window": 32768,
                "max_output_tokens": 8096
            })
            
        # Add V3 model (for coding)
        if provider_info.v3_model:
            models.append({
                "name": f"v3_chat",
                "description": f"{provider_info.name} Chat is for coding",
                "model_name": provider_info.v3_model,
                "model_type": "saas/openai",
                "base_url": provider_info.endpoint,
                "api_key": provider_info.api_key,
                "api_key_path": f"v3_chat",
                "is_reasoning": False,
                "input_price": provider_info.v3_input_price,
                "output_price": provider_info.v3_output_price,
                "average_speed": 0.0,
                "context_window": 32768,
                "max_output_tokens": 8096
            })
            
        return models
        
    def select_provider(self) -> Optional[Dict[str, Any]]:
        """
        Let user select a model provider and input necessary credentials.
        Returns a dictionary with provider info or None if cancelled.
        """
        

        result = radiolist_dialog(
            title=self.printer.get_message_from_key("model_provider_select_title"),
            text=self.printer.get_message_from_key("model_provider_select_text"),
            values=[
                ("volcano", self.printer.get_message_from_key("model_provider_volcano")),
                ("siliconflow", self.printer.get_message_from_key("model_provider_siliconflow")),
                ("deepseek", self.printer.get_message_from_key("model_provider_deepseek")),
                ("openrouter", self.printer.get_message_from_key("model_provider_openrouter"))
            ],
            style=dialog_style
        ).run()
        
        if result is None:
            return None

    
        provider_info = None
        for provider in PROVIDER_INFO_LIST:
            if provider.name == result:
                provider_info = provider
                break
        
        if provider_info is None:
            return None
        
        # if result == "volcano":
        #     # Get R1 endpoint
        #     r1_endpoint = input_dialog(
        #         title=self.printer.get_message_from_key("model_provider_api_key_title"),
        #         text=self.printer.get_message_from_key("model_provider_volcano_r1_text"),
        #         validator=VolcanoEndpointValidator(),
        #         style=dialog_style
        #     ).run()
            
        #     if r1_endpoint is None:
        #         return None
            
        #     provider_info.r1_model = r1_endpoint
            
        #     # Get V3 endpoint
        #     v3_endpoint = input_dialog(
        #         title=self.printer.get_message_from_key("model_provider_api_key_title"),
        #         text=self.printer.get_message_from_key("model_provider_volcano_v3_text"),
        #         validator=VolcanoEndpointValidator(),
        #         style=dialog_style
        #     ).run()
            
        #     if v3_endpoint is None:
        #         return None
                
        #     provider_info.v3_model = v3_endpoint
        
        # Get API key for all providers
        api_key = input_dialog(
            title=self.printer.get_message_from_key("model_provider_api_key_title"),
            text=self.printer.get_message_from_key(f"model_provider_{result}_api_key_text"),
            password=True,
            style=dialog_style
        ).run()
        
        if api_key is None:
            return None
            
        provider_info.api_key = api_key
        
        # 使用新的 LLMManager 添加模型
        models_to_add = self.to_models_json(provider_info)
        self.llm_manager.add_models(models_to_add)  # type: ignore
        
        self.printer.print_panel(
            self.printer.get_message_from_key("model_provider_selected"),
            text_options={"justify": "left"},
            panel_options={
                "title": self.printer.get_message_from_key("model_provider_success_title"),
                "border_style": "green"
            }
        )
        
        return provider_info.dict()


from collections import OrderedDict
from typing import Dict, List, Tuple, Any, Optional, Union
from dataclasses import dataclass, field
import re

from .parser import CommandParser
from .config import (
    ParserConfig,
    CommandConfig,
    ArgumentConfig,
    ArgType,
    ParamConstraint,
)


@dataclass
class ParsedCommand:
    """解析后的命令结果"""

    name: str
    args: List[Any] = field(default_factory=list)  # 转换后的类型
    kwargs: Dict[str, Any] = field(default_factory=dict)  # 转换后的类型
    raw_args: List[str] = field(default_factory=list)  # 原始字符串
    raw_kwargs: Dict[str, str] = field(default_factory=dict)  # 原始字符串
    remainder: Optional[str] = None  # 剩余参数
    errors: List[str] = field(default_factory=list)  # 解析错误

    @property
    def is_valid(self) -> bool:
        """是否有效（无错误）"""
        return len(self.errors) == 0


class TypedCommandParser(CommandParser):
    """
    支持类型和配置的命令解析器

    Example:
        >>> from autocoder.common.ac_style_command_parser import create_config
        >>>
        >>> # 创建配置
        >>> config = (create_config()
        ...     .command("model")
        ...         .positional("name", required=True, choices=["gpt-4", "gpt-3.5"])
        ...         .max_args(1)
        ...     .command("config")
        ...         .keyword("temperature", type=float, default=0.7)
        ...         .keyword("max_tokens", type=int)
        ...     .command("task-prefix")
        ...         .positional("prefix", required=True)
        ...         .collect_remainder("query")
        ...     .build()
        ... )
        >>>
        >>> # 创建解析器
        >>> parser = TypedCommandParser(config)
        >>>
        >>> # 解析命令
        >>> result = parser.parse_typed("/model gpt-4 /config temperature=0.8")
        >>> print(result["model"].args)  # ['gpt-4'] - 已验证
        >>> print(result["config"].kwargs)  # {'temperature': 0.8} - 已转换为float
    """

    def __init__(self, config: Optional[ParserConfig] = None):
        """
        初始化类型化命令解析器

        Args:
            config: 解析器配置，如果为None则使用默认配置
        """
        super().__init__()
        self.config = config or ParserConfig()

    def parse_typed(self, query: str) -> OrderedDict[str, ParsedCommand]:
        """
        解析命令并进行类型转换和验证

        Args:
            query: 命令行格式的查询字符串

        Returns:
            OrderedDict[str, ParsedCommand]: 解析和验证后的命令字典
        """
        # 使用基础解析器解析
        raw_commands = self.parse(query)

        # 如果配置了全局 collect_remainder，需要调整参数分配
        remainder_text = ""
        if self.config.collect_remainder and raw_commands:
            raw_commands, remainder_text = self._redistribute_args_for_global_remainder(
                raw_commands
            )

        # 处理每个命令
        typed_commands = OrderedDict()

        for cmd_name, cmd_info in raw_commands.items():
            raw_args = cmd_info["args"]
            raw_kwargs = cmd_info["kwargs"]

            # 获取命令配置
            cmd_config = self.config.get_command_config(cmd_name)

            if cmd_config:
                # 有配置，进行验证和转换
                parsed_cmd = self._process_configured_command(
                    cmd_name, cmd_config, raw_args, raw_kwargs
                )
            else:
                # 无配置
                if self.config.strict_mode:
                    # 严格模式下，未配置的命令是错误
                    parsed_cmd = ParsedCommand(
                        name=cmd_name,
                        raw_args=raw_args,
                        raw_kwargs=raw_kwargs,
                        errors=[f"Unknown command: {cmd_name}"],
                    )
                else:
                    # 非严格模式，保持原样
                    parsed_cmd = ParsedCommand(
                        name=cmd_name,
                        args=raw_args,
                        kwargs=raw_kwargs,
                        raw_args=raw_args,
                        raw_kwargs=raw_kwargs,
                    )

            typed_commands[cmd_name] = parsed_cmd

        # 如果有剩余文本，添加到特殊的 remainder 命令中
        if remainder_text:
            remainder_cmd = ParsedCommand(
                name=self.config.remainder_name,
                args=[remainder_text],
                kwargs={},
                raw_args=[remainder_text],
                raw_kwargs={},
                remainder=remainder_text,
            )
            typed_commands[self.config.remainder_name] = remainder_cmd

        return typed_commands

    def _process_configured_command(
        self,
        cmd_name: str,
        config: CommandConfig,
        raw_args: List[str],
        raw_kwargs: Dict[str, str],
    ) -> ParsedCommand:
        """
        处理有配置的命令

        Args:
            cmd_name: 命令名
            config: 命令配置
            raw_args: 原始位置参数
            raw_kwargs: 原始键值对参数

        Returns:
            ParsedCommand: 处理后的命令
        """
        parsed_cmd = ParsedCommand(
            name=cmd_name, raw_args=raw_args, raw_kwargs=raw_kwargs
        )

        # 验证参数
        valid, errors = config.validate_args(raw_args, raw_kwargs)
        if not valid:
            parsed_cmd.errors.extend(errors)

        # 处理位置参数
        converted_args = []
        remainder_args = []

        for i, arg_value in enumerate(raw_args):
            if i < len(config.positional_args):
                # 有对应的配置
                arg_config = config.positional_args[i]
                converted_value = self._convert_value(arg_value, arg_config.type)
                converted_args.append(converted_value)
            else:
                # 超出配置的参数
                if config.collect_remainder:
                    remainder_args.append(arg_value)
                else:
                    converted_args.append(arg_value)

        # 添加默认值
        for i in range(len(raw_args), len(config.positional_args)):
            arg_config = config.positional_args[i]
            if arg_config.default is not None:
                converted_args.append(arg_config.default)
            elif arg_config.constraint != ParamConstraint.REQUIRED:
                # 可选参数，不添加
                break

        parsed_cmd.args = converted_args

        # 处理剩余参数
        if config.collect_remainder and remainder_args:
            parsed_cmd.remainder = " ".join(remainder_args)

        # 处理键值对参数
        converted_kwargs = {}

        for key, value in raw_kwargs.items():
            if key in config.keyword_args:
                arg_config = config.keyword_args[key]
                converted_value = self._convert_value(value, arg_config.type)
                converted_kwargs[key] = converted_value
            else:
                # 未配置的键值对
                if not config.collect_remainder:
                    converted_kwargs[key] = value

        # 添加默认值
        for key, arg_config in config.keyword_args.items():
            if key not in converted_kwargs and arg_config.default is not None:
                converted_kwargs[key] = arg_config.default

        parsed_cmd.kwargs = converted_kwargs

        return parsed_cmd

    def _extract_remainder(self, raw_commands: OrderedDict, query: str) -> str:
        """
        提取剩余参数文本

        Args:
            raw_commands: 解析出的原始命令
            query: 原始查询字符串

        Returns:
            剩余参数文本
        """
        if not raw_commands:
            return ""

        # 获取最后一个命令
        last_cmd_name = list(raw_commands.keys())[-1]
        last_cmd_info = raw_commands[last_cmd_name]
        last_cmd_config = self.config.get_command_config(last_cmd_name)

        if not last_cmd_config:
            # 如果最后一个命令没有配置，将其所有参数作为剩余参数
            return " ".join(last_cmd_info["args"])

        # 根据最后一个命令的配置决定如何处理剩余参数
        raw_args = last_cmd_info["args"]
        raw_kwargs = last_cmd_info["kwargs"]

        remainder_args = []

        if last_cmd_config.arg_type in [ArgType.POSITIONAL, ArgType.MIXED]:
            # 如果有位置参数配置
            if last_cmd_config.positional_args:
                # 跳过已配置的位置参数数量
                configured_count = len(last_cmd_config.positional_args)
                if len(raw_args) > configured_count:
                    remainder_args = raw_args[configured_count:]
                elif len(raw_args) == configured_count and not raw_kwargs:
                    # 如果位置参数刚好匹配且没有键值对参数，检查是否还有未解析的文本
                    # 这种情况需要从原始查询中重新提取
                    return self._extract_text_after_last_command(query, raw_commands)
            else:
                # 没有位置参数配置，所有位置参数都是剩余参数
                remainder_args = raw_args

        if last_cmd_config.arg_type in [ArgType.KEYWORD, ArgType.MIXED]:
            # 如果是键值对类型，且没有位置参数配置，所有位置参数都是剩余参数
            if not last_cmd_config.positional_args:
                remainder_args = raw_args

        return " ".join(remainder_args)

    def _redistribute_args_for_global_remainder(
        self, raw_commands: OrderedDict
    ) -> Tuple[OrderedDict, str]:
        """
        为全局 collect_remainder 重新分配参数

        根据每个命令的配置限制其参数数量，收集多余的参数作为全局 remainder

        Args:
            raw_commands: 原始解析的命令

        Returns:
            Tuple[OrderedDict, str]: 调整后的命令字典和剩余参数文本
        """
        adjusted_commands = OrderedDict()
        all_remainder_args = []

        for cmd_name, cmd_info in raw_commands.items():
            raw_args = cmd_info["args"]
            raw_kwargs = cmd_info["kwargs"]
            cmd_config = self.config.get_command_config(cmd_name)

            if cmd_config:
                # 确定该命令应该接受的最大参数数量
                max_args = None

                if cmd_config.max_positional is not None:
                    # 明确设置了 max_positional
                    max_args = cmd_config.max_positional
                elif cmd_config.positional_args:
                    # 根据位置参数配置推断
                    max_args = len(cmd_config.positional_args)
                elif cmd_config.arg_type == ArgType.KEYWORD:
                    # 纯键值对命令，不接受位置参数
                    max_args = 0

                if max_args is not None and len(raw_args) > max_args:
                    # 保留前 max_args 个参数，其余的加入 remainder
                    kept_args = raw_args[:max_args]
                    remainder_args = raw_args[max_args:]
                    all_remainder_args.extend(remainder_args)

                    adjusted_commands[cmd_name] = {
                        "args": kept_args,
                        "kwargs": raw_kwargs,
                    }
                else:
                    # 参数数量在限制范围内，保持原样
                    adjusted_commands[cmd_name] = cmd_info
            else:
                # 没有配置，保持原样
                adjusted_commands[cmd_name] = cmd_info

        remainder_text = " ".join(all_remainder_args)
        return adjusted_commands, remainder_text

    def _extract_text_after_last_command(
        self, query: str, raw_commands: OrderedDict
    ) -> str:
        """
        从原始查询中提取最后一个命令之后的文本

        Args:
            query: 原始查询字符串
            raw_commands: 解析出的原始命令

        Returns:
            最后一个命令之后的剩余文本
        """
        if not raw_commands:
            return ""

        # 找到最后一个命令在查询字符串中的位置
        last_cmd_name = list(raw_commands.keys())[-1]
        last_cmd_pattern = rf"/{re.escape(last_cmd_name)}\b"

        # 找到最后一个命令的位置
        matches = list(re.finditer(last_cmd_pattern, query))
        if not matches:
            return ""

        last_match = matches[-1]
        cmd_end_pos = last_match.end()

        # 获取最后一个命令的参数
        last_cmd_info = raw_commands[last_cmd_name]
        remaining_text = query[cmd_end_pos:].strip()

        # 移除已经解析的参数
        for arg in last_cmd_info["args"]:
            # 简单的文本替换，可能需要更精确的处理
            arg_pattern = re.escape(arg)
            remaining_text = re.sub(
                rf"\b{arg_pattern}\b", "", remaining_text, 1
            ).strip()

        for key, value in last_cmd_info["kwargs"].items():
            kv_pattern = rf"\b{re.escape(key)}={re.escape(value)}\b"
            remaining_text = re.sub(kv_pattern, "", remaining_text).strip()

        return remaining_text.strip()

    def _convert_value(self, value: str, target_type: type) -> Any:
        """
        转换值的类型

        Args:
            value: 字符串值
            target_type: 目标类型

        Returns:
            转换后的值
        """
        if target_type == str:
            return value

        if target_type == bool:
            # 特殊处理布尔值
            if value.lower() in ("true", "1", "yes", "on"):
                return True
            elif value.lower() in ("false", "0", "no", "off"):
                return False
            else:
                return bool(value)

        try:
            return target_type(value)
        except (ValueError, TypeError):
            # 转换失败，返回原值
            return value

    def parse_with_config(
        self, query: str, config: ParserConfig
    ) -> OrderedDict[str, ParsedCommand]:
        """
        使用指定配置解析命令

        Args:
            query: 命令行格式的查询字符串
            config: 解析器配置

        Returns:
            OrderedDict[str, ParsedCommand]: 解析后的命令字典
        """
        old_config = self.config
        self.config = config
        try:
            return self.parse_typed(query)
        finally:
            self.config = old_config


class CommandWrapper:
    """
    命令包装器，提供对命令参数的便捷访问
    """

    def __init__(
        self, parsed_cmd: ParsedCommand, config: Optional[ParserConfig] = None
    ):
        self._parsed_cmd = parsed_cmd
        self._config = config

    def __getattr__(self, name: str) -> Any:
        """通过属性访问命令参数"""
        if name.startswith("_"):
            raise AttributeError(
                f"'{self.__class__.__name__}' object has no attribute '{name}'"
            )

        # 查找位置参数配置
        cmd_config = None
        if self._config:
            cmd_config = self._config.get_command_config(self._parsed_cmd.name)

        if cmd_config:
            # 如果有剩余参数且参数名匹配命令配置的 remainder_name
            if self._parsed_cmd.remainder and name == cmd_config.remainder_name:
                return self._parsed_cmd.remainder

            # 查找位置参数
            for i, arg_config in enumerate(cmd_config.positional_args):
                if arg_config.name == name and i < len(self._parsed_cmd.args):
                    return self._parsed_cmd.args[i]

            # 查找键值对参数
            if name in self._parsed_cmd.kwargs:
                return self._parsed_cmd.kwargs[name]

            # 如果键值对参数在配置中但不在 kwargs 中（用户没有提供），检查默认值
            if name in cmd_config.keyword_args:
                arg_config = cmd_config.keyword_args[name]
                # 如果有默认值，返回默认值；否则返回 None（因为它是可选的）
                return arg_config.default if arg_config.default is not None else None

        # 如果没有配置，尝试通过索引访问
        if name == "remainder" and self._parsed_cmd.remainder:
            return self._parsed_cmd.remainder

        # 回退到原始行为（用于没有配置的命令）
        if self._parsed_cmd.kwargs and name in self._parsed_cmd.kwargs:
            return self._parsed_cmd.kwargs[name]

        # 只有在没有任何配置的情况下，才使用单参数回退
        # 如果有配置但没有匹配到，应该抛出 AttributeError
        if not cmd_config:
            # 如果是简单的单参数情况，直接返回第一个参数
            if len(self._parsed_cmd.args) == 1 and not self._parsed_cmd.kwargs:
                return self._parsed_cmd.args[0]

        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{name}'"
        )

    def __repr__(self) -> str:
        return f"CommandWrapper({self._parsed_cmd.name})"


@dataclass
class TypedQueryCommand:
    """
    类型化的查询命令结果，提供便捷的属性访问

    与 QueryCommand 类似，但支持类型转换和验证
    """

    _commands: OrderedDict[str, ParsedCommand] = field(default_factory=OrderedDict)
    _config: Optional[ParserConfig] = None
    _query: str = ""

    def __getattr__(self, name: str) -> Any:
        """通过属性访问命令参数"""
        if name.startswith("_"):
            raise AttributeError(
                f"'{self.__class__.__name__}' object has no attribute '{name}'"
            )

        # 特殊处理 query 属性 - 优先从配置的 remainder_name 获取
        if name == "query":
            if self._config and self._config.collect_remainder:
                remainder_name = self._config.remainder_name
                if remainder_name in self._commands:
                    remainder_cmd = self._commands[remainder_name]
                    return remainder_cmd.remainder or ""
            return self._query

        # 查找命令
        parsed_cmd = self._get_command(name)
        if parsed_cmd:
            # 获取命令配置
            cmd_config = None
            if self._config:
                cmd_config = self._config.get_command_config(parsed_cmd.name)

            # 如果有配置（位置参数或键值对参数），检查是否需要 CommandWrapper
            if cmd_config and (cmd_config.positional_args or cmd_config.keyword_args):
                # 特殊处理：如果只有一个位置参数且参数名为 "value"，且没有剩余参数和键值对参数，直接返回值
                if (
                    len(cmd_config.positional_args) == 1
                    and cmd_config.positional_args[0].name == "value"
                    and len(parsed_cmd.args) == 1
                    and not parsed_cmd.remainder
                    and not parsed_cmd.kwargs
                ):
                    return parsed_cmd.args[0]
                # 其他情况返回 CommandWrapper 以支持嵌套访问
                return CommandWrapper(parsed_cmd, self._config)

            # 如果有剩余参数但没有位置参数配置，直接返回剩余参数
            if parsed_cmd.remainder:
                return parsed_cmd.remainder

            # 如果有键值对参数但没有配置，返回字典
            if parsed_cmd.kwargs:
                return parsed_cmd.kwargs

            # 如果有位置参数
            if parsed_cmd.args:
                return (
                    parsed_cmd.args[0] if len(parsed_cmd.args) == 1 else parsed_cmd.args
                )

            # 命令存在但没有参数
            return ""

        return None

    def _get_command(self, name: str) -> Optional[ParsedCommand]:
        """获取命令（支持连字符转换）"""
        # 直接查找
        if name in self._commands:
            return self._commands[name]

        # 尝试连字符转换
        hyphen_name = name.replace("_", "-")
        if hyphen_name in self._commands:
            return self._commands[hyphen_name]

        return None

    def has_command(self, command: str) -> bool:
        """检查是否包含指定命令"""
        return self._get_command(command) is not None

    def get_command(self, command: str) -> Optional[ParsedCommand]:
        """获取解析后的命令对象"""
        return self._get_command(command)

    def get_errors(self) -> Dict[str, List[str]]:
        """获取所有命令的错误"""
        errors = {}
        for cmd_name, parsed_cmd in self._commands.items():
            if parsed_cmd.errors:
                errors[cmd_name] = parsed_cmd.errors
        return errors

    def is_valid(self) -> bool:
        """检查是否所有命令都有效"""
        return all(cmd.is_valid for cmd in self._commands.values())

    def __repr__(self) -> str:
        """返回对象的字符串表示"""
        commands_str = ", ".join(self._commands.keys())
        valid_str = "valid" if self.is_valid() else "has errors"
        return f"TypedQueryCommand(commands=[{commands_str}], {valid_str})"


def parse_typed_query(
    query: str, config: Optional[ParserConfig] = None
) -> TypedQueryCommand:
    """
    解析类型化的查询命令

    Args:
        query: 命令行格式的查询字符串
        config: 解析器配置

    Returns:
        TypedQueryCommand: 类型化的查询命令对象

    Example:
        >>> from autocoder.common.ac_style_command_parser import create_config
        >>>
        >>> config = (create_config()
        ...     .command("model")
        ...         .positional("name", required=True)
        ...     .command("temperature")
        ...         .positional("value", type=float)
        ...     .build()
        ... )
        >>>
        >>> result = parse_typed_query("/model gpt-4 /temperature 0.8", config)
        >>> print(result.model)  # "gpt-4"
        >>> print(result.temperature)  # 0.8 (float类型)
        >>> print(result.is_valid())  # True
    """
    parser = TypedCommandParser(config)
    parsed_commands = parser.parse_typed(query)

    result = TypedQueryCommand()
    result._commands = parsed_commands
    result._config = config

    # 如果配置了全局 collect_remainder，query 会从特殊的 remainder 命令中获取
    # 否则，从第一个有剩余参数的命令中获取
    if config and config.collect_remainder:
        # query 会通过 __getattr__ 自动从 remainder_name 命令中获取
        pass
    else:
        # 兼容旧的行为：从第一个有剩余参数的命令中获取
        for cmd_name, parsed_cmd in parsed_commands.items():
            if parsed_cmd.remainder:
                result._query = parsed_cmd.remainder
                break

    return result



from enum import Enum

class AutoCommandStreamOutType(Enum):
    COMMAND_SUGGESTION = "command_suggestion"    
class IndexFilterStreamOutType(Enum):
    FILE_NUMBER_LIST = "file_number_list"

class AgenticFilterStreamOutType(Enum):
    AGENTIC_FILTER = "agentic_filter"


class CodeGenerateStreamOutType(Enum):
    CODE_GENERATE = "code_generate"

class CodeRankStreamOutType(Enum):
    CODE_RANK = "code_rank"

class LintStreamOutType(Enum):
    LINT = "lint"

class UnmergedBlocksStreamOutType(Enum):
    UNMERGED_BLOCKS = "unmerged_blocks"

class CompileStreamOutType(Enum):
    COMPILE = "compile"

class ContextMissingCheckStreamOutType(Enum):
    CONTEXT_MISSING_CHECK = "context_missing_check"

class IndexStreamOutType(Enum):
    INDEX_BUILD = "index_build"


"""
AutoCoder RAG SDK 工具函数

提供辅助函数，提升SDK的可用性。
"""

from typing import List


def format_contexts_for_display(
    contexts: List[str], max_length: int = 100
) -> List[str]:
    """
    格式化上下文用于显示

    Args:
        contexts: 上下文列表
        max_length: 每个上下文的最大显示长度

    Returns:
        格式化后的上下文列表
    """
    return [
        ctx[:max_length] + "..." if len(ctx) > max_length else ctx for ctx in contexts
    ]


def validate_doc_dir(doc_dir: str) -> bool:
    """
    验证文档目录是否有效

    Args:
        doc_dir: 文档目录路径

    Returns:
        是否有效
    """
    from pathlib import Path

    path = Path(doc_dir)
    return path.exists() and path.is_dir()


def print_response_summary(response) -> None:
    """
    打印响应摘要

    Args:
        response: RAGResponse 对象
    """
    print("=" * 60)
    print("RAG 查询结果")
    print("=" * 60)

    if response.success:
        print(f"\n✅ 查询成功")
        print(f"\n答案:\n{response.answer}")

        if response.contexts:
            print(f"\n参考文档 ({len(response.contexts)}个):")
            for i, ctx in enumerate(response.contexts[:3], 1):  # 只显示前3个
                preview = ctx[:150] + "..." if len(ctx) > 150 else ctx
                print(f"  {i}. {preview}")
            if len(response.contexts) > 3:
                print(f"  ... 还有 {len(response.contexts) - 3} 个文档")
    else:
        print(f"\n❌ 查询失败")
        print(f"错误: {response.error}")

    print("=" * 60)


def create_simple_config(doc_dir: str, **kwargs) -> "RAGConfig":
    """
    创建简化的配置对象

    快捷函数，避免导入 RAGConfig

    Args:
        doc_dir: 文档目录
        **kwargs: 其他配置参数

    Returns:
        RAGConfig 对象

    Example::

        config = create_simple_config("./docs", agentic=True, product_mode="pro")
    """
    from .models import RAGConfig

    return RAGConfig(doc_dir=doc_dir, **kwargs)


if __name__ == "__main__":
    # 测试工具函数
    contexts = ["这是一个很长的文档内容" * 10, "短文档"]
    formatted = format_contexts_for_display(contexts, max_length=50)
    print("格式化上下文测试:")
    for ctx in formatted:
        print(f"  - {ctx}")

    print(f"\n目录验证测试:")
    print(f"  当前目录有效: {validate_doc_dir('.')}")
    print(f"  无效目录: {validate_doc_dir('/nonexistent')}")



import json
import shlex
import fnmatch  # Add fnmatch for wildcard matching
from rich.console import Console
from rich.table import Table
import byzerllm
from autocoder.common.llms import LLMManager
from autocoder.common.printer import Printer
from autocoder.common.result_manager import ResultManager
from autocoder.common.model_speed_tester import render_speed_test_in_terminal
from autocoder.utils.llms import get_single_llm
from autocoder.common.core_config import get_memory_manager


def handle_models_command(query: str):
    """
    Handle /models subcommands:
      /models /list [pattern] - List all models (default + custom), optionally filter by pattern
      /models provider/model_name <api_key> - Activate a model with API key
      /models /add_provider name=xxx base_url=xxx ... - Add custom provider
      /models /remove <name> - Remove custom model by name
      /models /chat <model_name> <question> - Chat with a model
      /models /speed-test [rounds] - Test model speed
    """
    console = Console()
    printer = Printer(console=console)

    memory_manager = get_memory_manager()
    product_mode = memory_manager.get_config("product_mode", "lite")
    if product_mode != "lite":
        printer.print_in_terminal("models_lite_only", style="red")
        return

    # Check if the query is empty or only whitespace
    if not query.strip():
        printer.print_in_terminal("models_usage")
        return

    # Initialize LLM Manager
    llm_manager = LLMManager()

    subcmd = ""
    if "/list" in query:
        subcmd = "/list"
        query = query.replace("/list", "", 1).strip()

    if "/add_provider" in query:
        subcmd = "/add_provider"
        query = query.replace("/add_provider", "", 1).strip()

    if "/remove" in query:
        subcmd = "/remove"
        query = query.replace("/remove", "", 1).strip()

    if "/speed-test" in query:
        subcmd = "/speed-test"
        query = query.replace("/speed-test", "", 1).strip()

    if "/speed_test" in query:
        subcmd = "/speed-test"
        query = query.replace("/speed_test", "", 1).strip()

    if "input_price" in query:
        subcmd = "/input_price"
        query = query.replace("/input_price", "", 1).strip()

    if "output_price" in query:
        subcmd = "/output_price"
        query = query.replace("/output_price", "", 1).strip()

    if "/speed" in query:
        subcmd = "/speed"
        query = query.replace("/speed", "", 1).strip()

    if "/chat" in query:
        subcmd = "/chat"
        query = query.replace("/chat", "", 1).strip()

    # 如果没有找到任何子命令，检查是否是简化的激活命令格式: provider/model_name <api_key>
    if not subcmd and query.strip():
        parts = query.strip().split()
        if len(parts) >= 2 and "/" in parts[0]:
            # 这是简化的激活命令
            subcmd = "/activate"
        else:
            printer.print_in_terminal("models_usage")
            return

    if not subcmd:
        printer.print_in_terminal("models_usage")

    result_manager = ResultManager()

    if subcmd == "/list":
        pattern = query.strip()  # Get the filter pattern from the query

        # Get all models from LLM Manager
        all_models = llm_manager.get_all_models()
        models_list = list(all_models.values())

        if pattern:  # Apply filter if a pattern is provided
            filtered_models = [
                m for m in models_list if fnmatch.fnmatch(m.name, pattern)
            ]
        else:
            filtered_models = models_list

        if filtered_models:
            # Sort models by provider and name
            sorted_models = sorted(
                filtered_models, key=lambda x: (x.provider or "", x.name)
            )

            table = Table(
                title=printer.get_message_from_key("models_title")
                + (f" (Filtered by: '{pattern}')" if pattern else ""),
                expand=True,
                show_lines=True,
            )
            table.add_column(
                "Provider/Model", style="cyan", width=40, overflow="fold", no_wrap=False
            )
            table.add_column(
                "Description", style="white", width=30, overflow="fold", no_wrap=False
            )
            table.add_column(
                "Input Price (M)",
                style="magenta",
                width=15,
                overflow="fold",
                no_wrap=False,
            )
            table.add_column(
                "Output Price (M)",
                style="magenta",
                width=15,
                overflow="fold",
                no_wrap=False,
            )
            table.add_column(
                "API Key", style="blue", width=15, overflow="fold", no_wrap=False
            )

            for m in sorted_models:
                # Check if model has API key
                has_key = llm_manager.has_key(m.name)
                display_name = m.name
                if has_key:
                    display_name = f"{display_name} ✓"

                table.add_row(
                    display_name,
                    m.description or "",
                    f"{m.input_price:.2f}",
                    f"{m.output_price:.2f}",
                    "✓" if has_key else "✗",
                )
            console.print(table)

            # Convert models to dict for JSON serialization
            models_data = [
                {
                    "name": m.name,
                    "model_name": m.model_name,
                    "base_url": m.base_url,
                    "input_price": m.input_price,
                    "output_price": m.output_price,
                    "has_api_key": llm_manager.has_key(m.name),
                }
                for m in sorted_models
            ]

            result_manager.add_result(
                content=json.dumps(models_data, ensure_ascii=False),
                meta={"action": "models", "input": {"query": query}},
            )
        else:
            if pattern:
                printer.print_in_terminal(
                    "models_no_models_matching_pattern", style="yellow", pattern=pattern
                )
                result_manager.add_result(
                    content=f"No models found matching pattern: {pattern}",
                    meta={"action": "models", "input": {"query": query}},
                )
            else:
                printer.print_in_terminal("models_no_models", style="yellow")
                result_manager.add_result(
                    content="No models found",
                    meta={"action": "models", "input": {"query": query}},
                )

    elif subcmd == "/input_price":
        args = query.strip().split()
        if len(args) >= 2:
            name = args[0]
            try:
                price = float(args[1])
                if llm_manager.update_input_price(name, price):
                    printer.print_in_terminal(
                        "models_input_price_updated",
                        style="green",
                        name=name,
                        price=price,
                    )
                    result_manager.add_result(
                        content=f"models_input_price_updated: {name} {price}",
                        meta={"action": "models", "input": {"query": query}},
                    )
                else:
                    printer.print_in_terminal(
                        "models_not_found", style="red", name=name
                    )
                    result_manager.add_result(
                        content=f"models_not_found: {name}",
                        meta={"action": "models", "input": {"query": query}},
                    )
            except ValueError as e:
                result_manager.add_result(
                    content=f"models_invalid_price: {str(e)}",
                    meta={"action": "models", "input": {"query": query}},
                )
                printer.print_in_terminal(
                    "models_invalid_price", style="red", error=str(e)
                )
        else:
            result_manager.add_result(
                content=printer.get_message_from_key("models_input_price_usage"),
                meta={"action": "models", "input": {"query": query}},
            )
            printer.print_in_terminal("models_input_price_usage", style="red")

    elif subcmd == "/output_price":
        args = query.strip().split()
        if len(args) >= 2:
            name = args[0]
            try:
                price = float(args[1])
                if llm_manager.update_output_price(name, price):
                    printer.print_in_terminal(
                        "models_output_price_updated",
                        style="green",
                        name=name,
                        price=price,
                    )
                    result_manager.add_result(
                        content=f"models_output_price_updated: {name} {price}",
                        meta={"action": "models", "input": {"query": query}},
                    )
                else:
                    printer.print_in_terminal(
                        "models_not_found", style="red", name=name
                    )
                    result_manager.add_result(
                        content=f"models_not_found: {name}",
                        meta={"action": "models", "input": {"query": query}},
                    )
            except ValueError as e:
                printer.print_in_terminal(
                    "models_invalid_price", style="red", error=str(e)
                )
                result_manager.add_result(
                    content=f"models_invalid_price: {str(e)}",
                    meta={"action": "models", "input": {"query": query}},
                )
        else:
            result_manager.add_result(
                content=printer.get_message_from_key("models_output_price_usage"),
                meta={"action": "models", "input": {"query": query}},
            )
            printer.print_in_terminal("models_output_price_usage", style="red")

    elif subcmd == "/speed":
        args = query.strip().split()
        if len(args) >= 2:
            name = args[0]
            try:
                speed = float(args[1])
                # Speed functionality not implemented in LLMManager yet
                printer.print_in_terminal(
                    "models_speed_not_implemented", style="yellow"
                )
                result_manager.add_result(
                    content="Speed functionality not implemented yet",
                    meta={"action": "models", "input": {"query": query}},
                )
            except ValueError as e:
                printer.print_in_terminal(
                    "models_invalid_speed", style="red", error=str(e)
                )
                result_manager.add_result(
                    content=f"models_invalid_speed: {str(e)}",
                    meta={"action": "models", "input": {"query": query}},
                )
        else:
            result_manager.add_result(
                content=printer.get_message_from_key("models_speed_usage"),
                meta={"action": "models", "input": {"query": query}},
            )
            printer.print_in_terminal("models_speed_usage", style="red")

    elif subcmd == "/speed-test" or subcmd == "/check":
        test_rounds = 1  # 默认测试轮数

        enable_long_context = False
        if "/long_context" in query:
            enable_long_context = True
            query = query.replace("/long_context", "", 1).strip()

        if "/long-context" in query:
            enable_long_context = True
            query = query.replace("/long-context", "", 1).strip()

        # 解析可选的测试轮数参数
        args = query.strip().split()
        if args and args[0].isdigit():
            test_rounds = int(args[0])

        render_speed_test_in_terminal(
            product_mode, test_rounds, enable_long_context=enable_long_context
        )
        ## 等待优化，获取明细数据
        result_manager.add_result(
            content="models test success",
            meta={"action": "models", "input": {"query": query}},
        )

    elif subcmd == "/activate":
        # 简化的激活命令: /models provider/model_name <api_key>
        args = query.strip().split(" ")
        if len(args) >= 2:
            name, api_key = args[0], args[1]

            # 检查模型是否存在于内置模型中
            if not llm_manager.check_model_exists(name):
                printer.print_in_terminal("models_not_found", style="red", name=name)
                printer.print_in_terminal("models_not_found_hint", style="yellow")

                result_manager.add_result(
                    content=printer.get_message_from_key_with_format(
                        "models_not_found", name=name
                    )
                    + printer.get_message_from_key("models_not_found_hint"),
                    meta={"action": "models", "input": {"query": query}},
                )
                return

            result = llm_manager.update_model_with_api_key(name, api_key)
            if result:
                result_manager.add_result(
                    content=f"models_activated: {name}",
                    meta={"action": "models", "input": {"query": query}},
                )
                printer.print_in_terminal("models_added", style="green", name=name)

                # 检查 model 配置是否存在（合并视图：全局+项目），如果不存在则自动设置为刚激活的模型
                if not memory_manager.has_config("model", source="merged"):
                    memory_manager.set_config("model", name)
                    printer.print_in_terminal(
                        "models_auto_set_model", style="cyan", name=name
                    )
            else:
                result_manager.add_result(
                    content=f"models_activate_failed: {name}",
                    meta={"action": "models", "input": {"query": query}},
                )
                printer.print_in_terminal("models_add_failed", style="red", name=name)
        else:
            # 参数不足时的错误提示
            printer.print_in_terminal("models_activate_usage", style="red")
            printer.print_in_terminal("models_activate_example", style="yellow")
            printer.print_in_terminal("models_activate_list_hint", style="yellow")

            result_manager.add_result(
                content=printer.get_message_from_key("models_activate_usage")
                + "\n"
                + printer.get_message_from_key("models_activate_example")
                + printer.get_message_from_key("models_activate_list_hint"),
                meta={"action": "models", "input": {"query": query}},
            )

    elif subcmd == "/add_provider":
        # Parse key=value pairs: /models /add_model name=abc base_url=http://xx ...
        # Collect key=value pairs
        kv_pairs = shlex.split(query)
        data_dict = {}
        for pair in kv_pairs:
            if "=" not in pair:
                printer.print_in_terminal("models_add_model_params", style="red")
                continue
            k, v = pair.split("=", 1)
            data_dict[k.strip()] = v.strip()

        # Name is required
        if "name" not in data_dict:
            printer.print_in_terminal("models_add_model_name_required", style="red")
            return

        # Check duplication
        if llm_manager.check_model_exists(data_dict["name"]):
            printer.print_in_terminal(
                "models_add_model_exists", style="yellow", name=data_dict["name"]
            )
            result_manager.add_result(
                content=printer.get_message_from_key_with_format(
                    "models_add_model_exists", name=data_dict["name"]
                ),
                meta={"action": "models", "input": {"query": query}},
            )
            return

        # Create model with defaults
        final_model = {
            "name": data_dict["name"],
            "model_type": data_dict.get("model_type", "saas/openai"),
            "model_name": data_dict.get("model_name", data_dict["name"]),
            "base_url": data_dict.get("base_url", "https://api.openai.com/v1"),
            "provider": data_dict.get("provider", None),
            "api_key_path": data_dict.get("api_key_path", ""),
            "description": data_dict.get("description", ""),
            "is_reasoning": data_dict.get("is_reasoning", "false")
            in ["true", "True", "TRUE", "1"],
            "input_price": float(data_dict.get("input_price", "0.0")),
            "output_price": float(data_dict.get("output_price", "0.0")),
            "context_window": int(data_dict.get("context_window", "32768")),
            "max_output_tokens": int(data_dict.get("max_output_tokens", "8096")),
        }

        # Add API key if provided
        if "api_key" in data_dict:
            final_model["api_key"] = data_dict["api_key"]

        llm_manager.add_models([final_model])
        printer.print_in_terminal(
            "models_add_model_success", style="green", name=data_dict["name"]
        )
        result_manager.add_result(
            content=f"models_add_model_success: {data_dict['name']}",
            meta={"action": "models", "input": {"query": query}},
        )

    elif subcmd == "/remove":
        args = query.strip().split(" ")
        if len(args) < 1:
            printer.print_in_terminal("models_remove_usage", style="red")
            result_manager.add_result(
                content=printer.get_message_from_key("models_remove_usage"),
                meta={"action": "models", "input": {"query": query}},
            )
            return
        name = args[0]
        if not llm_manager.check_model_exists(name):
            printer.print_in_terminal(
                "models_add_model_remove", style="yellow", name=name
            )
            result_manager.add_result(
                content=printer.get_message_from_key_with_format(
                    "models_add_model_remove", name=name
                ),
                meta={"action": "models", "input": {"query": query}},
            )
            return

        # 尝试删除模型
        if llm_manager.remove_model(name):
            printer.print_in_terminal(
                "models_add_model_removed", style="green", name=name
            )
            result_manager.add_result(
                content=printer.get_message_from_key_with_format(
                    "models_add_model_removed", name=name
                ),
                meta={"action": "models", "input": {"query": query}},
            )
        else:
            # 删除失败，可能是默认模型
            printer.print_in_terminal("models_remove_failed", style="red", name=name)
            result_manager.add_result(
                content=f"Failed to remove model: {name}. Cannot remove default models.",
                meta={"action": "models", "input": {"query": query}},
            )

    elif subcmd == "/chat":
        if not query.strip():
            printer.print_in_terminal(
                "Please provide content in format: <model_name> <question>",
                style="yellow",
            )
            result_manager.add_result(
                content="Please provide content in format: <model_name> <question>",
                meta={"action": "models", "input": {"query": query}},
            )
            return

        # 分离模型名称和用户问题
        parts = query.strip().split(" ", 1)  # 只在第一个空格处分割
        if len(parts) < 2:
            printer.print_in_terminal(
                "Correct format should be: <model_name> <question>, where question can contain spaces",
                style="yellow",
            )
            result_manager.add_result(
                content="Correct format should be: <model_name> <question>, where question can contain spaces",
                meta={"action": "models", "input": {"query": query}},
            )
            return

        model_name = parts[0]
        user_question = parts[1]  # 这将包含所有剩余文本，保留空格
        memory_manager = get_memory_manager()
        product_mode = memory_manager.get_config("product_mode", "lite")

        try:
            # Get the model
            llm = get_single_llm(model_name, product_mode=product_mode)

            @byzerllm.prompt()
            def chat_func(content: str) -> str:
                """
                {{ content }}
                """
                return {}  # type: ignore

            # Support custom llm_config parameters using stream_chat_oai
            result = llm.stream_chat_oai(
                conversations=[{"role": "user", "content": user_question}],
                delta_mode=True,
            )

            output_text = ""
            in_thinking = False
            for chunk in result:
                reasoning_content = (
                    chunk[1].reasoning_content if chunk[1].reasoning_content else ""
                )
                content = chunk[0] if chunk[0] else ""

                if reasoning_content:
                    if not in_thinking:
                        print("<inner_thinking>", end="", flush=True)
                        output_text += "<inner_thinking>"
                        in_thinking = True
                    print(reasoning_content, end="", flush=True)
                    output_text += reasoning_content

                if content:
                    if in_thinking:
                        print("</inner_thinking>", end="", flush=True)
                        output_text += "</inner_thinking>"
                        in_thinking = False
                    print(content, end="", flush=True)
                    output_text += content

            if in_thinking:
                print("</inner_thinking>", end="", flush=True)
                output_text += "</inner_thinking>\n"

            print("\n")

            # Print the result

            result_manager.add_result(
                content=output_text,
                meta={"action": "models", "input": {"query": query}},
            )
        except Exception as e:
            error_message = f"Error chating with model: {str(e)}"
            printer.print_str_in_terminal(error_message, style="red")
            result_manager.add_result(
                content=error_message,
                meta={"action": "models", "input": {"query": query}},
            )
    else:
        printer.print_in_terminal(
            "models_unknown_subcmd", style="yellow", subcmd=subcmd
        )
        result_manager.add_result(
            content=printer.get_message_from_key_with_format(
                "models_unknown_subcmd", subcmd=subcmd
            ),
            meta={"action": "models", "input": {"query": query}},
        )



"""命令行参数解析模块"""

import argparse
import locale
import importlib.resources as resources
from autocoder.lang import lang_desc
from autocoder.version import __version__


def get_tokenizer_path():
    """获取默认的 tokenizer 路径"""
    try:
        tokenizer_path = str(resources.files("autocoder") / "data" / "tokenizer.json")
    except FileNotFoundError:
        tokenizer_path = None
    return tokenizer_path


def create_parser():
    """创建并配置参数解析器"""
    system_lang, _ = locale.getdefaultlocale()
    lang = "zh" if system_lang and system_lang.startswith("zh") else "en"
    desc = lang_desc[lang]
    tokenizer_path = get_tokenizer_path()

    parser = argparse.ArgumentParser(description="Auto Coder RAG Server")
    parser.add_argument(
        "--model_file", default="", help="Path to model configuration file"
    )
    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Build hybrid index command
    _add_build_index_parser(subparsers, desc, tokenizer_path)

    # Serve command
    _add_serve_parser(subparsers, desc, tokenizer_path)

    # Run command (直接运行RAG，不启动服务器)
    _add_run_parser(subparsers, desc, tokenizer_path)

    # Benchmark command
    _add_benchmark_parser(subparsers)

    # Tools command
    _add_tools_parser(subparsers, tokenizer_path)

    return parser


def _add_build_index_parser(subparsers, desc, tokenizer_path):
    """添加构建索引命令的参数解析器"""
    build_index_parser = subparsers.add_parser(
        "build_hybrid_index", help="Build hybrid index for RAG"
    )

    build_index_parser.add_argument(
        "--rag_storage_type",
        type=str,
        default="duckdb",
        help="The storage type of the RAG, duckdb or byzer-storage",
    )

    build_index_parser.add_argument(
        "--rag_index_build_workers",
        type=int,
        default=5,
        help="The number of workers to build the RAG index",
    )

    build_index_parser.add_argument(
        "--quick", action="store_true", help="Skip system initialization"
    )
    build_index_parser.add_argument("--file", default="", help=desc["file"])
    build_index_parser.add_argument("--model", default="v3_chat", help=desc["model"])
    build_index_parser.add_argument(
        "--model_file", default="", help="Path to model configuration file"
    )

    build_index_parser.add_argument("--on_ray", action="store_true", help="Run on Ray")

    build_index_parser.add_argument(
        "--index_model", default="", help=desc["index_model"]
    )
    build_index_parser.add_argument("--emb_model", default="", help=desc["emb_model"])
    build_index_parser.add_argument(
        "--ray_address", default="auto", help=desc["ray_address"]
    )
    build_index_parser.add_argument(
        "--required_exts", default="", help=desc["doc_build_parse_required_exts"]
    )
    build_index_parser.add_argument(
        "--source_dir", default=".", help="Source directory path"
    )
    build_index_parser.add_argument(
        "--tokenizer_path", default=tokenizer_path, help="Path to tokenizer file"
    )
    build_index_parser.add_argument(
        "--doc_dir", default="", help="Document directory path"
    )
    build_index_parser.add_argument(
        "--enable_hybrid_index", action="store_true", help="Enable hybrid index"
    )
    build_index_parser.add_argument(
        "--hybrid_index_max_output_tokens",
        type=int,
        default=1000000,
        help="The maximum number of tokens in the output. This is only used when enable_hybrid_index is true.",
    )


def _add_serve_parser(subparsers, desc, tokenizer_path):
    """添加服务命令的参数解析器"""
    serve_parser = subparsers.add_parser("serve", help="Start the RAG server")
    serve_parser.add_argument(
        "--quick", action="store_true", help="Skip system initialization"
    )
    serve_parser.add_argument("--file", default="", help=desc["file"])
    serve_parser.add_argument("--model", default="v3_chat", help=desc["model"])
    serve_parser.add_argument(
        "--model_file", default="", help="Path to model configuration file"
    )
    serve_parser.add_argument("--index_model", default="", help=desc["index_model"])
    serve_parser.add_argument("--ray_address", default="auto", help=desc["ray_address"])
    serve_parser.add_argument(
        "--index_filter_workers",
        type=int,
        default=100,
        help=desc["index_filter_workers"],
    )
    serve_parser.add_argument(
        "--index_filter_file_num",
        type=int,
        default=3,
        help=desc["index_filter_file_num"],
    )
    serve_parser.add_argument(
        "--rag_context_window_limit",
        type=int,
        default=56000,
        help="The input context window limit for RAG",
    )
    serve_parser.add_argument(
        "--full_text_ratio",
        type=float,
        default=0.7,
        help="The ratio of full text area in the input context window (0.0 to 1.0)",
    )
    serve_parser.add_argument(
        "--segment_ratio",
        type=float,
        default=0.2,
        help="The ratio of segment area in the input context window (0.0 to 1.0)",
    )
    serve_parser.add_argument(
        "--required_exts", default="", help=desc["doc_build_parse_required_exts"]
    )
    serve_parser.add_argument(
        "--rag_doc_filter_relevance", type=int, default=5, help=""
    )
    serve_parser.add_argument("--source_dir", default=".", help="")
    serve_parser.add_argument("--host", default="", help="")
    serve_parser.add_argument("--port", type=int, default=8000, help="")
    serve_parser.add_argument("--name", default="", help="RAG服务的名称（可选）")
    serve_parser.add_argument("--workers", type=int, default=4, help="")
    serve_parser.add_argument("--uvicorn_log_level", default="info", help="")
    serve_parser.add_argument("--allow_credentials", action="store_true", help="")
    serve_parser.add_argument("--allowed_origins", default=["*"], help="")
    serve_parser.add_argument("--allowed_methods", default=["*"], help="")
    serve_parser.add_argument("--allowed_headers", default=["*"], help="")
    serve_parser.add_argument("--api_key", default="", help="")
    serve_parser.add_argument("--served_model_name", default="", help="")
    serve_parser.add_argument("--prompt_template", default="", help="")
    serve_parser.add_argument("--ssl_keyfile", default="", help="")
    serve_parser.add_argument("--ssl_certfile", default="", help="")
    serve_parser.add_argument("--response_role", default="assistant", help="")
    serve_parser.add_argument(
        "--doc_dir",
        default="",
        help="Document directory path, also used as the root directory for serving static files",
    )
    serve_parser.add_argument(
        "--enable_local_image_host",
        action="store_true",
        help=" enable local image host for local Chat app",
    )
    serve_parser.add_argument(
        "--agentic", action="store_true", help="使用 AgenticRAG 而不是 LongContextRAG"
    )
    serve_parser.add_argument("--tokenizer_path", default=tokenizer_path, help="")
    serve_parser.add_argument(
        "--collections", default="", help="Collection name for indexing"
    )
    serve_parser.add_argument(
        "--base_dir",
        default="",
        help="Path where the processed text embeddings were stored",
    )
    serve_parser.add_argument(
        "--monitor_mode",
        action="store_true",
        help="Monitor mode for the doc update",
    )
    serve_parser.add_argument(
        "--max_static_path_length",
        type=int,
        default=3000,
        help="Maximum length allowed for static file paths (larger value to better support Chinese characters)",
    )
    serve_parser.add_argument(
        "--enable_nginx_x_accel",
        action="store_true",
        help="Enable Nginx X-Accel-Redirect for static file serving when behind Nginx",
    )
    serve_parser.add_argument(
        "--disable_auto_window",
        action="store_true",
        help="Disable automatic window adaptation for documents",
    )
    serve_parser.add_argument(
        "--disable_segment_reorder",
        action="store_true",
        help="Disable reordering of document segments after retrieval",
    )

    serve_parser.add_argument(
        "--disable_inference_enhance",
        action="store_true",
        help="Disable enhanced inference mode",
    )
    serve_parser.add_argument(
        "--inference_deep_thought",
        action="store_true",
        help="Enable deep thought in inference mode",
    )
    serve_parser.add_argument(
        "--inference_slow_without_deep_thought",
        action="store_true",
        help="Enable slow inference without deep thought",
    )
    serve_parser.add_argument(
        "--inference_compute_precision",
        type=int,
        default=64,
        help="The precision of the inference compute",
    )

    serve_parser.add_argument(
        "--enable_hybrid_index",
        action="store_true",
        help="Enable hybrid index",
    )

    serve_parser.add_argument(
        "--rag_storage_type",
        type=str,
        default="duckdb",
        help="The storage type of the RAG, duckdb or byzer-storage",
    )

    serve_parser.add_argument(
        "--hybrid_index_max_output_tokens",
        type=int,
        default=1000000,
        help="The maximum number of tokens in the output. This is only used when enable_hybrid_index is true.",
    )

    serve_parser.add_argument(
        "--without_contexts",
        action="store_true",
        help="Whether to return responses without contexts. only works when pro plugin is installed",
    )

    serve_parser.add_argument(
        "--data_cells_max_num",
        type=int,
        default=2000,
        help="Maximum number of data cells to process",
    )

    serve_parser.add_argument(
        "--product_mode",
        type=str,
        default="pro",
        help="The mode of the auto-coder.rag, lite/pro default is pro",
    )
    serve_parser.add_argument(
        "--lite",
        action="store_true",
        help="Run in lite mode (equivalent to --product_mode=lite)",
    )
    serve_parser.add_argument(
        "--pro",
        action="store_true",
        help="Run in pro mode (equivalent to --product_mode=pro)",
    )

    serve_parser.add_argument(
        "--recall_model",
        default="",
        help="The model used for recall documents",
    )

    serve_parser.add_argument(
        "--chunk_model",
        default="",
        help="The model used for chunk documents",
    )

    serve_parser.add_argument(
        "--qa_model",
        default="",
        help="The model used for question answering",
    )

    serve_parser.add_argument(
        "--emb_model",
        default="",
        help="The model used for embedding documents",
    )

    serve_parser.add_argument(
        "--agentic_model",
        default="",
        help="The model used for agentic operations",
    )

    serve_parser.add_argument(
        "--context_prune_model",
        default="",
        help="The model used for context pruning",
    )

    serve_parser.add_argument(
        "--firecrawl_api_key",
        default="",
        help="Firecrawl API key for web search and crawl functionality",
    )
    serve_parser.add_argument(
        "--metaso_api_key",
        default="",
        help="Metaso API key for web search and crawl functionality",
    )
    serve_parser.add_argument(
        "--bochaai_api_key",
        default="",
        help="BochaAI API key for web search and crawl functionality",
    )


def _add_run_parser(subparsers, desc, tokenizer_path):
    """添加 run 命令的参数解析器

    run 命令用于直接运行 RAG 查询，不启动服务器。
    支持从标准输入读取 prompt，例如: echo "问题" | auto-coder.rag run
    """
    run_parser = subparsers.add_parser(
        "run", help="Run RAG query directly without starting a server"
    )

    # 基本参数
    run_parser.add_argument("--model", default="v3_chat", help=desc["model"])
    run_parser.add_argument(
        "--model_file", default="", help="Path to model configuration file"
    )
    run_parser.add_argument("--doc_dir", required=True, help="Document directory path")
    run_parser.add_argument(
        "--query", default="", help="Query text (if not provided, read from stdin)"
    )

    # RAG 配置参数
    run_parser.add_argument(
        "--agentic", action="store_true", help="使用 AgenticRAG 而不是 LongContextRAG"
    )
    run_parser.add_argument(
        "--rag_context_window_limit",
        type=int,
        default=56000,
        help="The input context window limit for RAG",
    )
    run_parser.add_argument(
        "--full_text_ratio",
        type=float,
        default=0.7,
        help="The ratio of full text area in the input context window (0.0 to 1.0)",
    )
    run_parser.add_argument(
        "--segment_ratio",
        type=float,
        default=0.2,
        help="The ratio of segment area in the input context window (0.0 to 1.0)",
    )
    run_parser.add_argument(
        "--rag_doc_filter_relevance",
        type=int,
        default=5,
        help="Document relevance threshold",
    )
    run_parser.add_argument(
        "--required_exts", default="", help=desc["doc_build_parse_required_exts"]
    )

    run_parser.add_argument(
        "--index_filter_workers",
        type=int,
        default=100,
        help=desc["index_filter_workers"],
    )
    run_parser.add_argument(
        "--index_filter_file_num",
        type=int,
        default=3,
        help=desc["index_filter_file_num"],
    )

    # 模型配置
    run_parser.add_argument(
        "--tokenizer_path", default=tokenizer_path, help="Path to tokenizer file"
    )
    run_parser.add_argument(
        "--recall_model", default="", help="The model used for recall documents"
    )
    run_parser.add_argument(
        "--chunk_model", default="", help="The model used for chunk documents"
    )
    run_parser.add_argument(
        "--qa_model", default="", help="The model used for question answering"
    )
    run_parser.add_argument(
        "--emb_model", default="", help="The model used for embedding documents"
    )
    run_parser.add_argument(
        "--agentic_model", default="", help="The model used for agentic operations"
    )
    run_parser.add_argument(
        "--context_prune_model", default="", help="The model used for context pruning"
    )

    # 模式选择
    run_parser.add_argument(
        "--product_mode",
        type=str,
        default="lite",
        help="The mode of the auto-coder.rag, lite/pro default is lite for run command",
    )
    run_parser.add_argument(
        "--lite",
        action="store_true",
        help="Run in lite mode (equivalent to --product_mode=lite)",
    )
    run_parser.add_argument(
        "--pro",
        action="store_true",
        help="Run in pro mode (equivalent to --product_mode=pro)",
    )

    # 其他参数
    run_parser.add_argument("--ray_address", default="auto", help=desc["ray_address"])
    run_parser.add_argument(
        "--enable_hybrid_index",
        action="store_true",
        help="Enable hybrid index",
    )
    run_parser.add_argument(
        "--disable_auto_window",
        action="store_true",
        help="Disable automatic window adaptation for documents",
    )
    run_parser.add_argument(
        "--disable_segment_reorder",
        action="store_true",
        help="Disable reordering of document segments after retrieval",
    )
    run_parser.add_argument(
        "--hybrid_index_max_output_tokens",
        type=int,
        default=1000000,
        help="The maximum number of tokens in the output. This is only used when enable_hybrid_index is true.",
    )
    run_parser.add_argument(
        "--rag_storage_type",
        type=str,
        default="duckdb",
        help="The storage type of the RAG, duckdb or byzer-storage",
    )
    run_parser.add_argument(
        "--output_format",
        choices=["text", "json", "stream-json"],
        default="text",
        help="Output format: text (only answer), json (with metadata), or stream-json (streaming json output)",
    )

    # Web 搜索 API 密钥
    run_parser.add_argument(
        "--metaso_api_key",
        default="",
        help="Metaso API key for web search and crawl functionality (used in agentic mode)",
    )


def _add_benchmark_parser(subparsers):
    """添加基准测试命令的参数解析器"""
    benchmark_parser = subparsers.add_parser(
        "benchmark", help="Benchmark LLM client performance"
    )
    benchmark_parser.add_argument(
        "--model", default="v3_chat", help="Model to benchmark"
    )
    benchmark_parser.add_argument(
        "--model_file", default="", help="Path to model configuration file"
    )
    benchmark_parser.add_argument(
        "--parallel", type=int, default=10, help="Number of parallel requests"
    )
    benchmark_parser.add_argument(
        "--rounds", type=int, default=1, help="Number of rounds to run"
    )
    benchmark_parser.add_argument(
        "--type",
        choices=["openai", "byzerllm"],
        default="byzerllm",
        help="Client type to benchmark",
    )
    benchmark_parser.add_argument(
        "--api_key", default="", help="OpenAI API key for OpenAI client"
    )
    benchmark_parser.add_argument(
        "--base_url", default="", help="Base URL for OpenAI client"
    )
    benchmark_parser.add_argument(
        "--query", default="Hello, how are you?", help="Query to use for benchmarking"
    )


def _add_tools_parser(subparsers, tokenizer_path):
    """添加工具命令的参数解析器"""
    tools_parser = subparsers.add_parser("tools", help="Various tools")
    tools_subparsers = tools_parser.add_subparsers(dest="tool", help="Available tools")
    tools_parser.add_argument(
        "--product_mode",
        type=str,
        default="pro",
        help="The mode of the auto-coder.rag, lite/pro default is pro",
    )
    tools_parser.add_argument(
        "--lite",
        action="store_true",
        help="Run in lite mode (equivalent to --product_mode=lite)",
    )
    tools_parser.add_argument(
        "--pro",
        action="store_true",
        help="Run in pro mode (equivalent to --product_mode=pro)",
    )

    tools_parser.add_argument(
        "--model_file", default="", help="Path to model configuration file"
    )
    # Count tool
    count_parser = tools_subparsers.add_parser("count", help="Count tokens in a file")

    # Recall validation tool
    recall_parser = tools_subparsers.add_parser(
        "recall", help="Validate recall model performance"
    )
    recall_parser.add_argument(
        "--model", required=True, help="Model to use for recall validation"
    )
    recall_parser.add_argument(
        "--content", default=None, help="Content to validate against"
    )
    recall_parser.add_argument(
        "--query", default=None, help="Query to use for validation"
    )

    # Add chunk model validation tool
    chunk_parser = tools_subparsers.add_parser(
        "chunk", help="Validate chunk model performance"
    )
    chunk_parser.add_argument(
        "--model", required=True, help="Model to use for chunk validation"
    )
    chunk_parser.add_argument(
        "--content", default=None, help="Content to validate against"
    )
    chunk_parser.add_argument(
        "--query", default=None, help="Query to use for validation"
    )
    count_parser.add_argument(
        "--tokenizer_path",
        default=tokenizer_path,
        help="Path to the tokenizer",
    )
    count_parser.add_argument(
        "--file", required=True, help="Path to the file to count tokens"
    )
    count_parser.add_argument(
        "--output_format",
        default="text",
        choices=["text", "json"],
        help="Output format: text (rich table) or json",
    )


def parse_arguments(input_args=None):
    """解析命令行参数

    Returns:
        tuple: (args, parser, subparsers_dict) 其中 subparsers_dict 包含各个子命令的 parser
    """
    parser = create_parser()
    args = parser.parse_args(input_args)

    # 重新获取 subparsers 以便访问各个子命令的 parser
    # 这是为了支持 merge_args_with_config 需要访问特定子parser的默认值
    subparsers_actions = [
        action
        for action in parser._actions
        if isinstance(action, argparse._SubParsersAction)
    ]

    subparsers_dict = {}
    if subparsers_actions:
        subparsers_dict = subparsers_actions[0].choices

    return args, parser, subparsers_dict



import os
import json
import time
from typing import List, Dict, Optional, Any
from pydantic import BaseModel
from datetime import datetime

class MemoryItem(BaseModel):
    content: Any
    role: str

class MemoryEntry(BaseModel):
    timestamp: datetime
    conversation: List[MemoryItem]


def save_to_memory_file(ask_conversation,query:str,response:str):
    # Save to memory file                
    memory_dir = os.path.join(os.path.expanduser("~"), ".auto-coder", "memory")
    os.makedirs(memory_dir, exist_ok=True)
    memory_file = os.path.join(memory_dir, "memory.json")
    
    # Read existing memory or create new
    if os.path.exists(memory_file):
        with open(memory_file, 'r') as f:
            try:
                memory_data = json.load(f)
            except json.JSONDecodeError:
                memory_data = {}
    else:
        memory_data = {}
    
    # Add new conversation
    current_time = str(int(time.time()))
    memory_data[current_time] = [
        {"content": ask_conversation, "role": "background"},
        {"content": query, "role": "user"},
        {"content": response, "role": "assistant"}
    ]
    
    # Save memory
    with open(memory_file, 'w') as f:
        json.dump(memory_data, f, ensure_ascii=False, indent=2) 

    get_global_memory_file_paths()          
    tmp_dir = os.path.join(memory_dir, ".tmp")
    return tmp_dir

def load_from_memory_file() -> List[MemoryEntry]:
    """Load memory data from file and return as list of MemoryEntry objects"""
    memory_dir = os.path.join(os.path.expanduser("~"), ".auto-coder", "memory")
    memory_file = os.path.join(memory_dir, "memory.json")
    
    if not os.path.exists(memory_file):
        return []
    
    with open(memory_file, 'r') as f:
        try:
            memory_data = json.load(f)
        except json.JSONDecodeError:
            return []
    
    entries = []
    for timestamp_str, conversation in memory_data.items():
        try:
            timestamp = datetime.fromtimestamp(int(timestamp_str))
            memory_items = [MemoryItem(**item) for item in conversation]
            entries.append(MemoryEntry(
                timestamp=timestamp,
                conversation=memory_items
            ))
        except (ValueError, TypeError):
            continue
            
    # Sort entries by timestamp (oldest first)
    entries.sort(key=lambda x: x.timestamp)
    
    return entries

def get_global_memory_file_paths() -> List[str]:
    """Get global memory and generate temporary files in ~/.auto-coder/memory/.tmp
    
    Returns:
        List[str]: List of file paths for the generated temporary files
    """
    entries = load_from_memory_file()
    memory_dir = os.path.join(os.path.expanduser("~"), ".auto-coder", "memory")
    tmp_dir = os.path.join(memory_dir, ".tmp")
    os.makedirs(tmp_dir, exist_ok=True)
    
    file_paths = []
    
    for entry in entries:
        # Find assistant responses
        assistant_contents = [
            item.content for item in entry.conversation 
            if item.role == "assistant"
        ]
        
        if assistant_contents:
            timestamp_str = str(int(entry.timestamp.timestamp()))
            content = "\n".join(assistant_contents)
            file_path = os.path.join(tmp_dir, os.path.join("memory",f"{timestamp_str}.txt"))
            
            # Create parent directory if it doesn't exist
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            
            # Only write file if it doesn't exist
            if not os.path.exists(file_path):
                with open(file_path, "w", encoding="utf-8") as f:
                    f.write(content)
            
            file_paths.append(file_path)
    
    return file_paths




"""
Pull Request 统一管理模块

统一的 Pull Request 创建和管理模块，支持 GitHub、GitLab、Gitee、GitCode 四大代码托管平台的 PR 操作。
"""
from typing import Optional, List, Dict, Any
from loguru import logger

from .models import (
    PRConfig, PRResult, PRInfo, PRData, RepoInfo, PlatformType, 
    DEFAULT_TEMPLATES
)
from .manager import PullRequestManager, get_global_manager, set_global_config
from .exceptions import (
    PRError, AuthenticationError, RepositoryNotFoundError, 
    BranchNotFoundError, NetworkError, RateLimitError, 
    ValidationError, PlatformNotSupportedError, ConfigurationError
)
from .config import get_config
from .utils import (
    parse_git_url, detect_platform_from_repo, get_repo_info_from_path,
    get_current_branch, branch_exists, is_git_repo, get_default_remote_branch,
    ensure_branch_exists_remotely, is_main_branch, create_and_checkout_branch,
    generate_auto_branch_name, has_uncommitted_changes, auto_commit_changes,
    ensure_branch_has_commits
)


def create_pull_request(
    repo_path: str,
    title: str,
    source_branch: Optional[str] = None,
    target_branch: Optional[str] = None,
    description: str = "",
    labels: Optional[List[str]] = None,
    assignees: Optional[List[str]] = None,
    reviewers: Optional[List[str]] = None,
    draft: bool = False,
    template_type: Optional[str] = None,
    template_vars: Optional[Dict[str, str]] = None,
    platform: Optional[str] = None,
    token: Optional[str] = None,
    **kwargs
) -> PRResult:
    """
    创建 Pull Request（主要接口函数）
    
    Args:
        repo_path: 仓库路径
        title: PR标题
        source_branch: 源分支（可选，默认为当前分支）
        target_branch: 目标分支（可选，默认为远程默认分支或main）
        description: PR描述
        labels: 标签列表
        assignees: 负责人列表
        reviewers: 审查者列表
        draft: 是否为草稿PR
        template_type: 模板类型
        template_vars: 模板变量
        platform: 平台类型（可选，会自动检测）
        token: 访问令牌（可选）
        **kwargs: 其他配置参数
    
    Returns:
        PR创建结果
    """
    # 处理可选的分支参数
    if source_branch is None:
        source_branch = get_current_branch(repo_path)
        if not source_branch:
            raise ValidationError("无法获取当前分支，请指定source_branch参数")
    
    # 从当前分支，自动创建新分支
    auto_branch_name = generate_auto_branch_name()
    logger.info(f"检测到当前分支 '{source_branch}' 为主分支，自动创建新分支: {auto_branch_name}")
    
    if not create_and_checkout_branch(repo_path, auto_branch_name):
        raise ValidationError(f"无法从主分支 '{source_branch}' 创建新分支 '{auto_branch_name}'")
    
    source_branch = auto_branch_name
    logger.info(f"已切换到新分支: {source_branch}")
    
    if target_branch is None:
        target_branch = get_default_remote_branch(repo_path)
        if not target_branch:
            target_branch = "main"  # 默认值
    
    # 验证仓库是否为Git仓库
    if not is_git_repo(repo_path):
        raise ValidationError(f"路径 {repo_path} 不是一个有效的Git仓库")
    
    # 检查源分支和目标分支是否相同
    if source_branch == target_branch:
        raise ValidationError(f"源分支和目标分支不能相同: {source_branch}")
    
    # 确保分支有提交，如果没有则自动提交更改
    if not ensure_branch_has_commits(repo_path, source_branch, auto_commit=True):
        raise ValidationError(
            f"分支 '{source_branch}' 没有提交且无法自动提交更改。"
            f"请确保有需要提交的更改或手动提交后再创建PR。"
        )
    
    # 确保源分支存在于远程仓库，如果不存在则自动推送
    if not ensure_branch_exists_remotely(repo_path, source_branch):
        raise BranchNotFoundError(
            f"源分支 '{source_branch}' 在本地不存在或推送失败。"
            f"请检查分支是否存在或网络连接是否正常。"
        )
    
    # 检查目标分支是否存在于远程仓库
    if not branch_exists(repo_path, target_branch, remote=True):
        raise BranchNotFoundError(
            f"目标分支 '{target_branch}' 在远程仓库中不存在。"
        )
    
    # 如果提供了token，创建临时配置
    manager = get_global_manager()
    
    if token:
        # 自动检测平台或使用指定平台
        if not platform:
            detected_platform = detect_platform_from_repo(repo_path)
            if detected_platform:
                platform = detected_platform.value
            else:
                raise ValidationError("无法检测平台类型，请指定platform参数")
        
        # 创建临时配置
        temp_config = PRConfig(platform=PlatformType(platform), token=token)
        temp_manager = PullRequestManager(temp_config)
        
        return temp_manager.create_pull_request(
            repo_path=repo_path,
            source_branch=source_branch,
            target_branch=target_branch,
            title=title,
            description=description,
            labels=labels,
            assignees=assignees,
            reviewers=reviewers,
            draft=draft,
            template_type=template_type,
            template_vars=template_vars,
            platform=platform
        )
    else:
        # 使用全局管理器
        return manager.create_pull_request(
            repo_path=repo_path,
            source_branch=source_branch,
            target_branch=target_branch,
            title=title,
            description=description,
            labels=labels,
            assignees=assignees,
            reviewers=reviewers,
            draft=draft,
            template_type=template_type,
            template_vars=template_vars,
            platform=platform
        )


def get_pull_request(
    repo_path: str,
    pr_number: int,
    platform: Optional[str] = None,
    token: Optional[str] = None,
    **kwargs
) -> PRResult:
    """获取 Pull Request 信息"""
    manager = get_global_manager()
    
    if token:
        if not platform:
            detected_platform = detect_platform_from_repo(repo_path)
            if detected_platform:
                platform = detected_platform.value
            else:
                raise ValidationError("无法检测平台类型，请指定platform参数")
        
        config_data = {"token": token}
        config_data.update(kwargs)
        temp_config = PRConfig(platform=platform, **config_data)
        temp_manager = PullRequestManager(temp_config)
        
        return temp_manager.get_pull_request(repo_path, pr_number, platform)
    else:
        return manager.get_pull_request(repo_path, pr_number, platform)


def list_pull_requests(
    repo_path: str,
    state: str = "open",
    per_page: int = 30,
    page: int = 1,
    platform: Optional[str] = None,
    token: Optional[str] = None,
    **kwargs
) -> List[PRInfo]:
    """列出 Pull Requests"""
    manager = get_global_manager()
    
    if token:
        if not platform:
            detected_platform = detect_platform_from_repo(repo_path)
            if detected_platform:
                platform = detected_platform.value
            else:
                raise ValidationError("无法检测平台类型，请指定platform参数")
        
        config_data = {"token": token}
        config_data.update(kwargs)
        temp_config = PRConfig(platform=platform, **config_data)
        temp_manager = PullRequestManager(temp_config)
        
        return temp_manager.list_pull_requests(repo_path, state, per_page, page, platform)
    else:
        return manager.list_pull_requests(repo_path, state, per_page, page, platform)


# 导出所有公共接口
__all__ = [
    # 主要函数
    'create_pull_request',
    'get_pull_request', 
    'list_pull_requests',
    
    # 类和模型
    'PullRequestManager',
    'PRConfig',
    'PRResult',
    'PRInfo',
    'PRData',
    'RepoInfo',
    'PlatformType',
    
    # 异常
    'PRError',
    'AuthenticationError',
    'RepositoryNotFoundError',
    'BranchNotFoundError',
    'NetworkError',
    'RateLimitError',
    'ValidationError',
    'PlatformNotSupportedError',
    'ConfigurationError',
    
    # 工具函数
    'parse_git_url',
    'detect_platform_from_repo',
    'get_repo_info_from_path',
    'get_current_branch',
    'branch_exists',
    'is_git_repo',
    
    # 配置函数
    'get_config',
    'set_global_config',
    'get_global_manager',
    
    # 模板
    'DEFAULT_TEMPLATES'
]


from io import BytesIO
from pypdf import PdfReader
import traceback


def extract_text_from_pdf_old(file_path):
    with open(file_path, "rb") as f:
        pdf_content = f.read()
    pdf_file = BytesIO(pdf_content)
    pdf_reader = PdfReader(pdf_file)
    text = ""
    for page in pdf_reader.pages:
        text += page.extract_text()
    return text

def extract_text_from_pdf(file_path, llm=None, product_mode="lite"):
    try:        
        from autocoder.utils._markitdown import MarkItDown
        md_converter = MarkItDown(llm=llm, product_mode=product_mode)
        result = md_converter.convert(file_path)
        return result.text_content
    except (BaseException, Exception) as e:
        traceback.print_exc()
        return extract_text_from_pdf_old(file_path)
